REPOSITORY_URL = "https://pkgs.safetycli.com/repository"

PYPI_PUBLIC_REPOSITORY_URL = f"{REPOSITORY_URL}/public/pypi/simple/"
PYPI_ORGANIZATION_REPOSITORY_URL = f"{REPOSITORY_URL}/{{}}/pypi/simple/"
PYPI_PROJECT_REPOSITORY_URL = f"{REPOSITORY_URL}/{{}}/project/{{}}/pypi/simple/"

NPMJS_PUBLIC_REPOSITORY_URL = f"{REPOSITORY_URL}/public/npmjs/"
NPMJS_ORGANIZATION_REPOSITORY_URL = f"{REPOSITORY_URL}/{{}}/npmjs/"
NPMJS_PROJECT_REPOSITORY_URL = f"{REPOSITORY_URL}/{{}}/project/{{}}/npmjs/"

PROJECT_CONFIG = ".safety-project.ini"

MOST_FREQUENTLY_DOWNLOADED_PYPI_PACKAGES = (
    "boto3",
    "urllib3",
    "botocore",
    "requests",
    "setuptools",
    "certifi",
    "idna",
    "charset-normalizer",
    "aiobotocore",
    "typing-extensions",
    "python-dateutil",
    "s3transfer",
    "packaging",
    "grpcio-status",
    "s3fs",
    "six",
    "fsspec",
    "pyyaml",
    "numpy",
    "importlib-metadata",
    "cryptography",
    "zipp",
    "cffi",
    "pip",
    "pandas",
    "google-api-core",
    "pycparser",
    "pydantic",
    "protobuf",
    "wheel",
    "jmespath",
    "attrs",
    "rsa",
    "pyasn1",
    "click",
    "platformdirs",
    "pytz",
    "colorama",
    "jinja2",
    "awscli",
    "markupsafe",
    "tomli",
    "pyjwt",
    "googleapis-common-protos",
    "filelock",
    "virtualenv",
    "cachetools",
    "wrapt",
    "google-auth",
    "pluggy",
    "pytest",
    "pydantic-core",
    "pyparsing",
    "docutils",
    "pyarrow",
    "pyasn1-modules",
    "requests-oauthlib",
    "aiohttp",
    "scipy",
    "jsonschema",
    "oauthlib",
    "sqlalchemy",
    "iniconfig",
    "exceptiongroup",
    "yarl",
    "decorator",
    "multidict",
    "psutil",
    "soupsieve",
    "greenlet",
    "tzdata",
    "pillow",
    "isodate",
    "pygments",
    "beautifulsoup4",
    "annotated-types",
    "requests-toolbelt",
    "frozenlist",
    "tomlkit",
    "pyopenssl",
    "aiosignal",
    "distlib",
    "async-timeout",
    "more-itertools",
    "openpyxl",
    "tqdm",
    "et-xmlfile",
    "grpcio",
    "deprecated",
    "cloudpickle",
    "lxml",
    "pynacl",
    "werkzeug",
    "proto-plus",
    "azure-core",
    "google-cloud-storage",
    "asn1crypto",
    "coverage",
    "websocket-client",
    "msgpack",
    "h11",
    "rich",
    "dill",
    "pexpect",
    "sniffio",
    "anyio",
    "mypy-extensions",
    "ptyprocess",
    "importlib-resources",
    "sortedcontainers",
    "matplotlib",
    "chardet",
    "rpds-py",
    "grpcio-tools",
    "aiohappyeyeballs",
    "flask",
    "httpx",
    "referencing",
    "scikit-learn",
    "jsonschema-specifications",
    "httpcore",
    "pyzmq",
    "poetry-core",
    "keyring",
    "google-cloud-core",
    "python-dotenv",
    "pathspec",
    "markdown-it-py",
    "pkginfo",
    "msal",
    "networkx",
    "bcrypt",
    "mdurl",
    "gitpython",
    "psycopg2-binary",
    "poetry-plugin-export",
    "google-resumable-media",
    "paramiko",
    "kiwisolver",
    "smmap",
    "gitdb",
    "xmltodict",
    "snowflake-connector-python",
    "tabulate",
    "cycler",
    "typedload",
    "jaraco-classes",
    "jeepney",
    "secretstorage",
    "ruamel-yaml",
    "tenacity",
    "wcwidth",
    "build",
    "backoff",
    "shellingham",
    "threadpoolctl",
    "regex",
    "itsdangerous",
    "portalocker",
    "py",
    "google-crc32c",
    "rapidfuzz",
    "pyproject-hooks",
    "py4j",
    "google-cloud-bigquery",
    "fastjsonschema",
    "sqlparse",
    "mccabe",
    "pytest-cov",
    "awswrangler",
    "trove-classifiers",
    "msal-extensions",
    "azure-storage-blob",
    "google-api-python-client",
    "pycodestyle",
    "joblib",
    "google-auth-oauthlib",
    "ruamel-yaml-clib",
    "tzlocal",
    "docker",
    "alembic",
    "fonttools",
    "prompt-toolkit",
    "cachecontrol",
    "azure-identity",
    "distro",
    "marshmallow",
    "uritemplate",
    "isort",
    "cython",
    "ply",
    "httplib2",
    "redis",
    "pymysql",
    "pyrsistent",
    "gym-notices",
    "google-auth-httplib2",
    "poetry",
    "blinker",
    "defusedxml",
    "dnspython",
    "dulwich",
    "toml",
    "gunicorn",
    "crashtest",
    "markdown",
    "nest-asyncio",
    "babel",
    "cleo",
    "sentry-sdk",
    "opentelemetry-api",
    "scramp",
    "multiprocess",
    "installer",
    "termcolor",
    "black",
    "huggingface-hub",
    "mock",
    "msrest",
    "pendulum",
    "requests-aws4auth",
    "ipython",
    "pyflakes",
    "pycryptodomex",
    "grpc-google-iam-v1",
    "types-requests",
    "azure-common",
    "traitlets",
    "fastapi",
    "setuptools-scm",
    "tornado",
    "flake8",
    "contourpy",
    "prometheus-client",
    "future",
    "openai",
    "mako",
    "pycryptodome",
    "imageio",
    "jedi",
    "webencodings",
    "pygithub",
    "parso",
    "transformers",
    "typing-inspect",
    "kubernetes",
    "jsonpointer",
    "matplotlib-inline",
    "starlette",
    "loguru",
    "opentelemetry-sdk",
    "retry",
    "argcomplete",
    "pkgutil-resolve-name",
    "redshift-connector",
    "elasticsearch",
    "pymongo",
    "opentelemetry-semantic-conventions",
    "pytzdata",
    "pytest-runner",
    "asgiref",
    "pg8000",
    "bs4",
    "datadog",
    "debugpy",
    "python-json-logger",
    "jsonpath-ng",
    "uvicorn",
    "executing",
    "smart-open",
    "zope-interface",
    "asttokens",
    "typer",
    "aioitertools",
    "apache-airflow",
    "sagemaker",
    "arrow",
    "google-pasta",
    "pyspark",
    "humanfriendly",
    "websockets",
    "stack-data",
    "shapely",
    "pure-eval",
    "torch",
    "oscrypto",
    "tokenizers",
    "pysocks",
    "sphinx",
    "typeguard",
    "tox",
    "scikit-image",
    "requests-file",
    "google-cloud-pubsub",
    "pytest-mock",
    "google-cloud-secret-manager",
    "snowflake-sqlalchemy",
    "mysql-connector-python",
    "pylint",
    "jupyter-core",
    "jupyter-client",
    "astroid",
    "jsonpatch",
    "setproctitle",
    "adal",
    "types-python-dateutil",
    "ipykernel",
    "xgboost",
    "orjson",
    "schema",
    "tb-nightly",
    "nbconvert",
    "xlrd",
    "toolz",
    "appdirs",
    "aiofiles",
    "sympy",
    "opensearch-py",
    "nodeenv",
    "pywavelets",
    "jaraco-functools",
    "jupyter-server",
    "nbformat",
    "jupyterlab",
    "progressbar2",
    "comm",
    "identify",
    "bleach",
    "mypy",
    "pathos",
    "pyodbc",
    "pre-commit",
    "xlsxwriter",
    "rfc3339-validator",
    "aws-requests-auth",
    "gym",
    "pox",
    "ppft",
    "mistune",
    "aenum",
    "jaraco-context",
    "tinycss2",
    "pbr",
    "google-cloud-appengine-logging",
    "notebook",
    "db-dtypes",
    "mpmath",
    "sentencepiece",
    "responses",
    "cfgv",
    "cattrs",
    "python-utils",
    "slack-sdk",
    "jupyterlab-server",
    "nbclient",
    "lz4",
    "ipywidgets",
    "sshtunnel",
    "absl-py",
    "widgetsnbextension",
    "watchdog",
    "asynctest",
    "semver",
    "rfc3986",
    "google-cloud-aiplatform",
    "jupyterlab-widgets",
    "altair",
    "pandas-gbq",
    "click-man",
    "tensorboard",
    "smdebug-rulesconfig",
    "simplejson",
    "text-unidecode",
    "argon2-cffi",
    "apache-airflow-providers-common-sql",
    "snowballstemmer",
    "azure-mgmt-core",
    "docker-pycreds",
    "nltk",
    "python-slugify",
    "croniter",
    "structlog",
    "selenium",
    "antlr4-python3-runtime",
    "google-cloud-logging",
    "argon2-cffi-bindings",
    "azure-storage-file-datalake",
    "django",
    "pydeequ",
    "pytest-xdist",
    "h5py",
    "google-cloud-resource-manager",
    "dataclasses",
    "execnet",
    "send2trash",
    "opentelemetry-proto",
    "google-cloud-bigquery-storage",
    "oauth2client",
    "dataclasses-json",
    "json5",
    "tiktoken",
    "wandb",
    "databricks-sql-connector",
    "langchain-core",
    "overrides",
    "prettytable",
    "pandocfilters",
    "semantic-version",
    "jupyterlab-pygments",
    "msrestazure",
    "safetensors",
    "hvac",
    "colorlog",
    "imbalanced-learn",
    "monotonic",
    "seaborn",
    "alabaster",
    "terminado",
    "webcolors",
    "ordered-set",
    "graphql-core",
    "notebook-shim",
    "lazy-object-proxy",
    "funcsigs",
    "numba",
    "llvmlite",
    "gremlinpython",
    "xxhash",
    "great-expectations",
    "flatbuffers",
    "pydata-google-auth",
    "fqdn",
    "uri-template",
    "imagesize",
    "opentelemetry-exporter-otlp-proto-common",
    "isoduration",
    "backports-tarfile",
    "wsproto",
    "tensorflow",
    "thrift",
    "hypothesis",
    "rfc3986-validator",
    "trio",
    "inflection",
    "html5lib",
    "plotly",
    "entrypoints",
    "sphinxcontrib-serializinghtml",
    "jupyter-events",
    "lockfile",
    "coloredlogs",
    "sphinxcontrib-htmlhelp",
    "cached-property",
    "sphinxcontrib-qthelp",
    "sphinxcontrib-devhelp",
    "sphinxcontrib-applehelp",
    "gast",
    "azure-cli",
    "azure-datalake-store",
    "opentelemetry-exporter-otlp-proto-http",
    "pyproject-api",
    "azure-mgmt-resource",
    "async-lru",
    "faker",
    "sphinxcontrib-jsmath",
    "nose",
    "opencv-python",
    "outcome",
    "statsmodels",
    "readme-renderer",
    "jupyter-server-terminals",
    "libcst",
    "retrying",
    "datasets",
    "aniso8601",
    "pybind11",
    "databricks-sdk",
    "pyroaring",
    "azure-keyvault-secrets",
    "email-validator",
    "argparse",
    "parameterized",
    "docopt",
    "google-cloud-audit-log",
    "confluent-kafka",
    "kafka-python",
    "pymssql",
    "zeep",
    "gcsfs",
    "click-plugins",
    "jupyter-lsp",
    "ruff",
    "deepdiff",
    "docstring-parser",
    "tblib",
    "time-machine",
    "jiter",
    "patsy",
    "azure-storage-common",
    "deprecation",
    "azure-nspkg",
    "databricks-cli",
    "nh3",
    "twine",
    "invoke",
    "delta-spark",
    "watchtower",
    "mlflow",
    "pydantic-settings",
    "azure-mgmt-storage",
    "opentelemetry-exporter-otlp-proto-grpc",
    "applicationinsights",
    "dbt-core",
    "freezegun",
    "pickleshare",
    "apache-airflow-providers-ssh",
    "python-multipart",
    "langchain",
    "uv",
    "unidecode",
    "azure-keyvault-keys",
    "azure-cosmos",
    "pytest-metadata",
    "pipenv",
    "tensorboard-data-server",
    "azure-graphrbac",
    "google-cloud-kms",
    "backcall",
    "trio-websocket",
    "azure-keyvault",
    "pytest-asyncio",
    "psycopg2",
    "google-cloud-dataproc",
    "keras",
    "datetime",
    "zope-event",
    "apache-airflow-providers-google",
    "backports-zoneinfo",
    "google-cloud-monitoring",
    "looker-sdk",
    "azure-mgmt-containerregistry",
    "makefun",
    "google-cloud-vision",
    "mlflow-skinny",
    "hatchling",
    "spacy",
    "torchvision",
    "apache-airflow-providers-snowflake",
    "google-cloud-spanner",
    "google-cloud-container",
    "nvidia-nccl-cu12",
    "triton",
    "gevent",
    "google-cloud-dlp",
    "uvloop",
    "simple-salesforce",
    "tldextract",
    "analytics-python",
    "apache-airflow-providers-databricks",
    "tensorflow-estimator",
    "google-cloud-bigquery-datatransfer",
    "azure-mgmt-keyvault",
    "azure-mgmt-cosmosdb",
    "azure-mgmt-compute",
    "graphviz",
    "google-cloud-tasks",
    "ujson",
    "opentelemetry-instrumentation",
    "azure-mgmt-authorization",
    "fastavro",
    "httptools",
    "pathlib2",
    "azure-mgmt-network",
    "google-cloud-datacatalog",
    "pkce",
    "google-ads",
    "opt-einsum",
    "sh",
    "jsondiff",
    "azure-mgmt-msi",
    "google-cloud-firestore",
    "evergreen-py",
    "google-cloud-bigtable",
    "astunparse",
    "watchfiles",
    "configparser",
    "flask-appbuilder",
    "fabric",
    "azure-mgmt-recoveryservices",
    "apache-airflow-providers-mysql",
    "scp",
    "db-contrib-tool",
    "google-cloud-build",
    "omegaconf",
    "azure-mgmt-monitor",
    "ecdsa",
    "gspread",
    "azure-mgmt-signalr",
    "azure-mgmt-containerinstance",
    "blis",
    "thinc",
    "bitarray",
    "murmurhash",
    "pycrypto",
    "dask",
    "requests-mock",
    "catalogue",
    "cymem",
    "azure-mgmt-sql",
    "preshed",
    "google-cloud-workflows",
    "opentelemetry-exporter-otlp",
    "azure-mgmt-web",
    "google-cloud-redis",
    "azure-batch",
    "kombu",
    "pywin32",
    "azure-data-tables",
    "wasabi",
    "azure-mgmt-containerservice",
    "azure-mgmt-servicebus",
    "azure-mgmt-redis",
    "google-cloud-dataplex",
    "srsly",
    "pytimeparse",
    "google-cloud-language",
    "authlib",
    "google-cloud-automl",
    "google-cloud-videointelligence",
    "google-cloud-os-login",
    "azure-mgmt-rdbms",
    "brotli",
    "pyserial",
    "azure-mgmt-dns",
    "langchain-community",
    "nvidia-cudnn-cu12",
    "texttable",
    "azure-mgmt-advisor",
    "google-cloud-memcache",
    "azure-mgmt-eventhub",
    "tensorflow-serving-api",
    "gsutil",
    "lark",
    "azure-cli-core",
    "flask-cors",
    "pysftp",
    "celery",
    "langcodes",
    "azure-mgmt-batch",
    "azure-mgmt-loganalytics",
    "azure-mgmt-cdn",
    "ninja",
    "azure-mgmt-recoveryservicesbackup",
    "azure-mgmt-iothub",
    "azure-mgmt-search",
    "azure-mgmt-marketplaceordering",
    "azure-mgmt-trafficmanager",
    "azure-mgmt-managementgroups",
    "pip-tools",
    "azure-mgmt-cognitiveservices",
    "azure-mgmt-devtestlabs",
    "azure-mgmt-eventgrid",
    "python-gnupg",
    "jira",
    "pypdf2",
    "azure-mgmt-applicationinsights",
    "azure-mgmt-servicefabric",
    "billiard",
    "azure-mgmt-media",
    "azure-mgmt-billing",
    "ratelimit",
    "azure-mgmt-iothubprovisioningservices",
    "azure-mgmt-policyinsights",
    "azure-mgmt-nspkg",
    "google-cloud-orchestration-airflow",
    "apache-airflow-providers-cncf-kubernetes",
    "azure-mgmt-batchai",
    "azure-mgmt-iotcentral",
    "azure-mgmt-datamigration",
    "graphql-relay",
    "azure-mgmt-maps",
    "graphene",
    "azure-appconfiguration",
    "amqp",
    "google-cloud-dataproc-metastore",
    "mdit-py-plugins",
    "google-cloud-translate",
    "ijson",
    "sqlalchemy-bigquery",
    "vine",
    "nvidia-cublas-cu12",
    "nvidia-nvjitlink-cu12",
    "spacy-loggers",
    "spacy-legacy",
    "levenshtein",
    "agate",
    "azure-mgmt-datalake-nspkg",
    "knack",
    "yapf",
    "awscrt",
    "azure-mgmt-datalake-store",
    "google-cloud-dataform",
    "types-pyyaml",
    "confection",
    "propcache",
    "google-cloud-speech",
    "nvidia-cuda-runtime-cu12",
    "opencensus",
    "opencensus-context",
    "nvidia-cuda-cupti-cu12",
    "nvidia-cuda-nvrtc-cu12",
    "parsedatetime",
    "nvidia-cusparse-cu12",
    "nvidia-cufft-cu12",
    "nvidia-cusolver-cu12",
    "grpcio-gcp",
    "nvidia-curand-cu12",
    "google-cloud-texttospeech",
    "typing",
    "humanize",
    "pytest-html",
    "langsmith",
    "nvidia-nvtx-cu12",
    "flask-sqlalchemy",
    "opentelemetry-util-http",
    "narwhals",
    "azure-multiapi-storage",
    "gcloud-aio-storage",
    "pycountry",
    "jsonpickle",
    "zstandard",
    "avro-python3",
    "libclang",
    "apispec",
    "gcloud-aio-auth",
    "azure-storage-queue",
    "contextlib2",
    "azure-mgmt-datalake-analytics",
    "gcloud-aio-bigquery",
    "azure-mgmt-reservations",
    "javaproperties",
    "tensorflow-io-gcs-filesystem",
    "azure-loganalytics",
    "djangorestframework",
    "azure-mgmt-consumption",
    "hpack",
    "google-cloud-compute",
    "click-didyoumean",
    "azure-mgmt-relay",
    "parsimonious",
    "azure-synapse-artifacts",
    "python-magic",
    "azure-cli-telemetry",
    "click-repl",
    "moto",
    "pyathena",
    "pyproj",
    "protobuf3-to-dict",
    "durationpy",
    "stevedore",
    "python-daemon",
    "azure-synapse-spark",
    "apache-airflow-providers-http",
    "mypy-boto3-s3",
    "pyspnego",
    "cfn-lint",
    "astor",
    "azure-mgmt-apimanagement",
    "h2",
    "hyperframe",
    "azure-mgmt-hdinsight",
    "azure-mgmt-privatedns",
    "boto3-stubs",
    "mashumaro",
    "dateparser",
    "ml-dtypes",
    "mysqlclient",
    "azure-mgmt-security",
    "opencensus-ext-azure",
    "azure-mgmt-synapse",
    "azure-mgmt-kusto",
    "azure-mgmt-netapp",
    "grpcio-health-checking",
    "azure-mgmt-redhatopenshift",
    "iso8601",
    "lightgbm",
    "azure-mgmt-appconfiguration",
    "azure-keyvault-administration",
    "boto",
    "azure-mgmt-sqlvirtualmachine",
    "azure-mgmt-imagebuilder",
    "azure-synapse-accesscontrol",
    "enum34",
    "azure-mgmt-servicelinker",
    "azure-mgmt-botservice",
    "azure-mgmt-servicefabricmanagedclusters",
    "jpype1",
    "python-jose",
    "azure-mgmt-databoxedge",
    "azure-synapse-managedprivateendpoints",
    "azure-mgmt-extendedlocation",
    "office365-rest-python-client",
    "onnxruntime",
    "azure-mgmt-managedservices",
    "cramjam",
    "urllib3-secure-extra",
    "avro",
    "holidays",
    "psycopg",
    "botocore-stubs",
    "fasteners",
    "resolvelib",
    "partd",
    "hyperlink",
    "leather",
    "apscheduler",
    "flask-wtf",
    "jupyter",
    "marisa-trie",
    "locket",
    "jupyter-console",
    "python-http-client",
    "elastic-transport",
    "dbt-extractor",
    "tensorflow-text",
    "language-data",
    "inflect",
    "fuzzywuzzy",
    "cytoolz",
    "cmake",
    "parse",
    "python-gitlab",
    "mypy-boto3-rds",
    "tifffile",
    "eth-utils",
    "eth-hash",
    "netaddr",
    "incremental",
    "setuptools-rust",
    "python-levenshtein",
    "geopandas",
    "twisted",
    "langchain-text-splitters",
    "types-awscrt",
    "apache-airflow-providers-fab",
    "yamllint",
    "cligj",
    "sphinx-rtd-theme",
    "azure-mgmt-deploymentmanager",
    "pytest-timeout",
    "lazy-loader",
    "wtforms",
    "bytecode",
    "accelerate",
    "polars",
    "sendgrid",
    "frozendict",
    "flask-login",
    "opentelemetry-instrumentation-requests",
    "jaydebeapi",
    "eth-typing",
    "dacite",
    "types-pytz",
    "py-cpuinfo",
    "querystring-parser",
    "universal-pathlib",
    "dbt-semantic-interfaces",
    "magicattr",
    "cssselect",
    "fastparquet",
    "opencv-python-headless",
    "automat",
    "unicodecsv",
    "constantly",
    "kfp",
    "ddtrace",
    "logbook",
    "envier",
    "cloudpathlib",
    "types-s3transfer",
    "google-cloud-dataflow-client",
    "sqlalchemy-utils",
    "apache-beam",
    "validators",
    "bracex",
    "apache-airflow-providers-ftp",
    "phonenumbers",
    "diskcache",
    "mergedeep",
    "slicer",
    "shap",
    "python-docx",
    "types-urllib3",
    "pytest-rerunfailures",
    "types-setuptools",
    "pathy",
    "pytz-deprecation-shim",
    "yappi",
    "pydot",
    "types-protobuf",
    "ipython-genutils",
    "pytorch-lightning",
    "fire",
    "apache-airflow-providers-sqlite",
    "nvidia-cublas-cu11",
    "azure-storage-file-share",
    "mmh3",
    "azure-mgmt-datafactory",
    "azure-servicebus",
    "nvidia-cudnn-cu11",
    "inject",
    "typed-ast",
    "connexion",
    "configargparse",
    "linkify-it-py",
    "aws-sam-translator",
    "slackclient",
    "eth-abi",
    "pydash",
    "timm",
    "datadog-api-client",
    "nvidia-cuda-runtime-cu11",
    "nvidia-cuda-nvrtc-cu11",
    "geographiclib",
    "gradio",
    "cron-descriptor",
    "ansible",
    "azure-kusto-data",
    "django-cors-headers",
    "junit-xml",
    "geopy",
    "uc-micro-py",
    "pyee",
    "xarray",
    "ansible-core",
    "pypdf",
    "pyotp",
    "starkbank-ecdsa",
    "geoip2",
    "multimethod",
    "eth-account",
    "meson",
    "jellyfish",
    "futures",
    "cachelib",
    "flask-caching",
    "natsort",
    "autopep8",
    "torchaudio",
    "torchmetrics",
    "pydub",
    "pandera",
    "pyhcl",
    "apache-airflow-providers-slack",
    "oracledb",
    "google-cloud-run",
    "h3",
    "apache-airflow-providers-amazon",
    "sqlalchemy-spanner",
    "events",
    "google-cloud-batch",
    "requests-ntlm",
    "bottle",
    "google-cloud-storage-transfer",
    "junitparser",
    "apache-airflow-providers-smtp",
    "apache-airflow-providers-imap",
    "emoji",
    "crcmod",
    "statsd",
    "limits",
    "apache-airflow-providers-common-io",
    "methodtools",
    "asyncpg",
    "strictyaml",
    "wcmatch",
    "marshmallow-sqlalchemy",
    "faiss-cpu",
    "sentence-transformers",
    "psycopg-binary",
    "azure-keyvault-certificates",
    "django-filter",
    "maxminddb",
    "weasel",
    "gql",
    "onnx",
    "fiona",
    "boltons",
    "dbt-common",
    "bidict",
    "keras-applications",
    "json-merge-patch",
    "elasticsearch-dsl",
    "ftfy",
    "swagger-ui-bundle",
    "tableauserverclient",
    "flask-jwt-extended",
    "lightning-utilities",
    "meson-python",
    "google-cloud",
)

TOP_NPMJS_PACKAGES = (
    "tslib",
    "inherits",
    "safe-buffer",
    "ms",
    "function-bind",
    "hasown",
    "graceful-fs",
    "debug",
    "string_decoder",
    "ansi-regex",
    "readable-stream",
    "strip-ansi",
    "wrappy",
    "balanced-match",
    "js-tokens",
    "util-deprecate",
    "isarray",
    "ansi-styles",
    "brace-expansion",
    "has-symbols",
    "once",
    "minimatch",
    "es-errors",
    "gopd",
    "es-define-property",
    "call-bind-apply-helpers",
    "dunder-proto",
    "es-object-atoms",
    "get-intrinsic",
    "math-intrinsics",
    "get-proto",
    "concat-map",
    "lodash",
    "core-util-is",
    "color-name",
    "semver",
    "supports-color",
    "color-convert",
    "safer-buffer",
    "loose-envify",
    "picocolors",
    "chalk",
    "is-fullwidth-code-point",
    "mime-db",
    "string-width",
    "minimist",
    "glob",
    "mime-types",
    "object-assign",
    "inflight",
    "path-is-absolute",
    "iconv-lite",
    "has-flag",
    "uuid",
    "call-bound",
    "escape-string-regexp",
    "fs.realpath",
    "process-nextick-args",
    "has-tostringtag",
    "source-map",
    "signal-exit",
    "emoji-regex",
    "postcss",
    "lru-cache",
    "mkdirp",
    "delayed-stream",
    "nanoid",
    "qs",
    "yallist",
    "react",
    "punycode",
    "combined-stream",
    "caniuse-lite",
    "source-map-js",
    "@types/node",
    "form-data",
    "wrap-ansi",
    "scheduler",
    "react-dom",
    "@aws-sdk/types",
    "asynckit",
    "commander",
    "base64-js",
    "has-property-descriptors",
    "@smithy/types",
    "define-data-property",
    "object-inspect",
    "set-function-length",
    "fast-deep-equal",
    "side-channel-map",
    "isexe",
    "side-channel-list",
    "side-channel-weakmap",
    "call-bind",
    "side-channel",
    "which",
    "is-core-module",
    "rimraf",
    "ieee754",
    "resolve",
    "path-parse",
    "supports-preserve-symlinks-flag",
    "ajv",
    "undici-types",
    "json-schema-traverse",
    "@babel/helper-validator-identifier",
    "buffer",
    "xtend",
    "is-number",
    "async",
    "is-stream",
    "es-set-tostringtag",
    "is-arrayish",
    "mime",
    "@swc/helpers",
    "streamsearch",
    "core-js",
    "fill-range",
    "busboy",
    "fast-json-stable-stringify",
    "is-extglob",
    "path-exists",
    "follow-redirects",
    "braces",
    "mimic-fn",
    "uri-js",
    "os-tmpdir",
    "is-glob",
    "end-of-stream",
    "cross-spawn",
    "shebang-regex",
    "react-is",
    "onetime",
    "is-callable",
    "webidl-conversions",
    "extend",
    "tr46",
    "to-regex-range",
    "camelcase",
    "find-up",
    "@babel/runtime",
    "client-only",
    "styled-jsx",
    "next",
    "@next/env",
    "@next/swc-darwin-arm64",
    "@next/swc-darwin-x64",
    "@next/swc-linux-x64-gnu",
    "@next/swc-win32-x64-msvc",
    "@next/swc-linux-arm64-gnu",
    "@next/swc-linux-arm64-musl",
    "@next/swc-linux-x64-musl",
    "@next/swc-win32-arm64-msvc",
    "path-key",
    "shebang-command",
    "@next/swc-win32-ia32-msvc",
    "whatwg-url",
    "type-fest",
    "possible-typed-array-names",
    "universalify",
    "p-limit",
    "assert-plus",
    "for-each",
    "available-typed-arrays",
    "object-keys",
    "jsonfile",
    "statuses",
    "picomatch",
    "node-fetch",
    "sprintf-js",
    "is-buffer",
    "which-typed-array",
    "jsbn",
    "depd",
    "@babel/types",
    "cliui",
    "y18n",
    "@babel/helper-string-parser",
    "kind-of",
    "micromatch",
    "glob-parent",
    "yargs",
    "negotiator",
    "tunnel-agent",
    "p-locate",
    "is-regex",
    "ee-first",
    "globals",
    "esprima",
    "yargs-parser",
    "argparse",
    "is-typed-array",
    "@babel/parser",
    "locate-path",
    "fs-extra",
    "@babel/code-frame",
    "setprototypeof",
    "pify",
    "clone",
    "http-errors",
    "esutils",
    "tough-cookie",
    "escape-html",
    "is-typedarray",
    "get-stream",
    "error-ex",
    "prop-types",
    "ansi-escapes",
    "on-finished",
    "parse-json",
    "tweetnacl",
    "acorn",
    "ini",
    "toidentifier",
    "json-stringify-safe",
    "minipass",
    "axios",
    "bytes",
    "@jridgewell/sourcemap-codec",
    "path-type",
    "bl",
    "unpipe",
    "asn1",
    "has-ansi",
    "buffer-from",
    "p-try",
    "get-caller-file",
    "regenerator-runtime",
    "mute-stream",
    "require-directory",
    "json-schema",
    "normalize-path",
    "safe-regex-test",
    "jsesc",
    "fresh",
    "escalade",
    "path-to-regexp",
    "through",
    "performance-now",
    "define-properties",
    "parseurl",
    "destroy",
    "restore-cursor",
    "ws",
    "json5",
    "@jridgewell/resolve-uri",
    "encodeurl",
    "@jridgewell/trace-mapping",
    "js-yaml",
    "cookie",
    "range-parser",
    "cli-cursor",
    "extsprintf",
    "tmp",
    "etag",
    "rxjs",
    "vary",
    "isstream",
    "accepts",
    "content-type",
    "caseless",
    "media-typer",
    "psl",
    "decamelize",
    "pump",
    "raw-body",
    "verror",
    "forever-agent",
    "http-signature",
    "content-disposition",
    "oauth-sign",
    "send",
    "agent-base",
    "dashdash",
    "hosted-git-info",
    "spdx-license-ids",
    "ecc-jsbn",
    "type-is",
    "spdx-expression-parse",
    "getpass",
    "aws-sign2",
    "bcrypt-pbkdf",
    "typed-array-buffer",
    "imurmurhash",
    "strip-json-comments",
    "@jridgewell/set-array",
    "jsprim",
    "sshpk",
    "@jridgewell/gen-mapping",
    "request",
    "is-generator-function",
    "methods",
    "isobject",
    "sax",
    "finalhandler",
    "set-blocking",
    "serve-static",
    "ipaddr.js",
    "convert-source-map",
    "strip-bom",
    "har-validator",
    "spdx-exceptions",
    "spdx-correct",
    "resolve-from",
    "aws4",
    "slash",
    "number-is-nan",
    "entities",
    "utils-merge",
    "forwarded",
    "cli-width",
    "csstype",
    "cookie-signature",
    "fsevents",
    "validate-npm-package-license",
    "run-async",
    "object.assign",
    "body-parser",
    "merge-descriptors",
    "inquirer",
    "proxy-addr",
    "har-schema",
    "normalize-package-data",
    "https-proxy-agent",
    "estraverse",
    "@smithy/is-array-buffer",
    "make-dir",
    "is-plain-object",
    "external-editor",
    "express",
    "@babel/template",
    "bluebird",
    "json-parse-even-better-errors",
    "chownr",
    "indent-string",
    "@babel/generator",
    "is-date-object",
    "ignore",
    "chardet",
    "@babel/traverse",
    "anymatch",
    "babel-runtime",
    "typedarray",
    "npm-run-path",
    "decode-uri-component",
    "code-point-at",
    "@smithy/util-buffer-from",
    "events",
    "array-flatten",
    "readdirp",
    "is-extendable",
    "@smithy/property-provider",
    "pinkie",
    "@smithy/protocol-http",
    "repeat-string",
    "through2",
    "log-symbols",
    "aproba",
    "functions-have-names",
    "chokidar",
    "execa",
    "set-function-name",
    "regexp.prototype.flags",
    "concat-stream",
    "pinkie-promise",
    "@aws-crypto/util",
    "binary-extensions",
    "nan",
    "is-symbol",
    "@smithy/util-utf8",
    "domelementtype",
    "@babel/helper-module-imports",
    "asap",
    "dotenv",
    "defaults",
    "@smithy/shared-ini-file-loader",
    "callsites",
    "source-map-support",
    "electron-to-chromium",
    "browserslist",
    "is-plain-obj",
    "extend-shallow",
    "@nodelib/fs.stat",
    "is-string",
    "lines-and-columns",
    "has-bigints",
    "is-binary-path",
    "domutils",
    "component-emitter",
    "is-boolean-object",
    "is-number-object",
    "is-bigint",
    "dom-serializer",
    "abbrev",
    "moment",
    "merge2",
    "which-boxed-primitive",
    "tar",
    "array-union",
    "internal-slot",
    "fast-glob",
    "is-shared-array-buffer",
    "@smithy/node-config-provider",
    "domhandler",
    "is-array-buffer",
    "is-map",
    "is-set",
    "is-weakmap",
    "is-weakset",
    "which-collection",
    "stop-iteration-iterator",
    "array-buffer-byte-length",
    "has-proto",
    "globby",
    "wcwidth",
    "figures",
    "@smithy/util-uri-escape",
    "minimalistic-assert",
    "es-to-primitive",
    "for-in",
    "read-pkg",
    "globalthis",
    "is-arguments",
    "process",
    "bowser",
    "cli-spinners",
    "os-homedir",
    "es-abstract",
    "merge-stream",
    "node-releases",
    "is-negative-zero",
    "randombytes",
    "@smithy/util-hex-encoding",
    "queue-microtask",
    "@babel/helper-plugin-utils",
    "reusify",
    "string.prototype.trimend",
    "string.prototype.trimstart",
    "is-windows",
    "update-browserslist-db",
    "eventemitter3",
    "nopt",
    "unbox-primitive",
    "get-symbol-description",
    "is-weakref",
    "@smithy/util-middleware",
    "function.prototype.name",
    "ora",
    "is-data-view",
    "fastq",
    "retry",
    "is-async-function",
    "async-function",
    "safe-push-apply",
    "is-finalizationregistry",
    "own-keys",
    "string.prototype.trim",
    "safe-array-concat",
    "typed-array-length",
    "typed-array-byte-length",
    "arraybuffer.prototype.slice",
    "typed-array-byte-offset",
    "data-view-buffer",
    "data-view-byte-length",
    "data-view-byte-offset",
    "which-builtin-type",
    "reflect.getprototypeof",
    "set-proto",
    "http-proxy-agent",
    "is-accessor-descriptor",
    "is-data-descriptor",
    "deep-extend",
    "bn.js",
    "import-fresh",
    "read-pkg-up",
    "run-parallel",
    "util",
    "p-finally",
    "is-descriptor",
    "htmlparser2",
    "repeat-element",
    "arr-diff",
    "atob",
    "arr-flatten",
    "ret",
    "delegates",
    "wordwrap",
    "array-unique",
    "@smithy/querystring-builder",
    "@aws-sdk/util-utf8-browser",
    "@nodelib/fs.scandir",
    "colors",
    "big.js",
    "define-property",
    "@nodelib/fs.walk",
    "write-file-atomic",
    "@smithy/abort-controller",
    "expand-brackets",
    "@smithy/util-base64",
    "is-wsl",
    "source-map-resolve",
    "source-map-url",
    "resolve-url",
    "extglob",
    "urix",
    "is-unicode-supported",
    "yaml",
    "boolbase",
    "arr-union",
    "assign-symbols",
    "@smithy/querystring-parser",
    "setimmediate",
    "nth-check",
    "postcss-value-parser",
    "bindings",
    "@swc/counter",
    "minizlib",
    "proxy-from-env",
    "fast-levenshtein",
    "get-value",
    "dir-glob",
    "@aws-crypto/supports-web-crypto",
    "@aws-sdk/util-locate-window",
    "has-unicode",
    "invariant",
    "css-what",
    "remove-trailing-separator",
    "require-main-filename",
    "deepmerge",
    "promise",
    "file-uri-to-path",
    "has-values",
    "set-value",
    "map-cache",
    "cosmiconfig",
    "@smithy/url-parser",
    "prelude-ls",
    "@aws-crypto/sha256-js",
    "lower-case",
    "parent-module",
    "safe-regex",
    "pkg-dir",
    "neo-async",
    "require-from-string",
    "to-object-path",
    "normalize-url",
    "object-visit",
    "split-string",
    "which-module",
    "has-value",
    "css-select",
    "copy-descriptor",
    "map-visit",
    "@smithy/fetch-http-handler",
    "@smithy/node-http-handler",
    "object-copy",
    "ci-info",
    "gauge",
    "type-check",
    "co",
    "is-interactive",
    "union-value",
    "collection-visit",
    "are-we-there-yet",
    "unset-value",
    "static-extend",
    "mixin-deep",
    "pascalcase",
    "deep-is",
    "stream-shift",
    "cache-base",
    "hoist-non-react-statics",
    "class-utils",
    "@smithy/middleware-serde",
    "object.pick",
    "use",
    "levn",
    "base",
    "word-wrap",
    "@babel/helper-validator-option",
    "regex-not",
    "regjsparser",
    "to-regex",
    "wide-align",
    "snapdragon",
    "loader-utils",
    "@babel/compat-data",
    "@smithy/util-stream",
    "xmlbuilder",
    "optionator",
    "emojis-list",
    "fs-minipass",
    "console-control-strings",
    "commondir",
    "fragment-cache",
    "@babel/helper-compilation-targets",
    "@babel/helper-module-transforms",
    "snapdragon-util",
    "regenerate",
    "posix-character-classes",
    "to-buffer",
    "encoding",
    "npmlog",
    "no-case",
    "load-json-file",
    "snapdragon-node",
    "esrecurse",
    "nanomatch",
    "regjsgen",
    "@smithy/middleware-stack",
    "parse5",
    "@aws-sdk/middleware-user-agent",
    "@types/json-schema",
    "@aws-crypto/sha256-browser",
    "whatwg-fetch",
    "lowercase-keys",
    "@aws-sdk/util-user-agent-node",
    "hash.js",
    "@aws-sdk/util-user-agent-browser",
    "@aws-sdk/middleware-logger",
    "@aws-sdk/middleware-host-header",
    "is-obj",
    "uglify-js",
    "@smithy/middleware-endpoint",
    "fast-xml-parser",
    "rc",
    "text-table",
    "arrify",
    "@babel/helper-annotate-as-pure",
    "@babel/helpers",
    "diff",
    "brorand",
    "strnum",
    "underscore",
    "prepend-http",
    "@babel/core",
    "prettier",
    "classnames",
    "json-buffer",
    "p-map",
    "strip-indent",
    "gensync",
    "@smithy/smithy-client",
    "pseudomap",
    "pako",
    "eastasianwidth",
    "sha.js",
    "color-string",
    "cssesc",
    "got",
    "keyv",
    "regexpu-core",
    "http-cache-semantics",
    "@aws-sdk/util-endpoints",
    "yocto-queue",
    "elliptic",
    "requires-port",
    "duplexify",
    "mimic-response",
    "q",
    "hoek",
    "ajv-keywords",
    "postcss-selector-parser",
    "tar-stream",
    "eslint-scope",
    "is-utf8",
    "minimalistic-crypto-utils",
    "on-headers",
    "hmac-drbg",
    "@ampproject/remapping",
    "color",
    "@aws-sdk/middleware-recursion-detection",
    "escodegen",
    "strip-final-newline",
    "lodash.debounce",
    "@smithy/util-body-length-browser",
    "es6-promise",
    "url",
    "decompress-response",
    "human-signals",
    "is-finite",
    "at-least-node",
    "deep-equal",
    "acorn-walk",
    "array-uniq",
    "redent",
    "strip-eof",
    "@babel/plugin-syntax-jsx",
    "to-fast-properties",
    "buffer-crc32",
    "is-path-inside",
    "simple-swizzle",
    "he",
    "typescript",
    "is-docker",
    "repeating",
    "next-tick",
    "es5-ext",
    "find-cache-dir",
    "get-stdin",
    "ripemd160",
    "tapable",
    "@smithy/util-config-provider",
    "es6-symbol",
    "foreground-child",
    "es6-iterator",
    "amdefine",
    "json-parse-better-errors",
    "event-emitter",
    "interpret",
    "schema-utils",
    "cipher-base",
    "d",
    "fs-constants",
    "lodash.isplainobject",
    "fast-uri",
    "@aws-sdk/client-sso",
    "eslint-visitor-keys",
    "path-scurry",
    "ecdsa-sig-formatter",
    "ssri",
    "hash-base",
    "unique-slug",
    "buffer-equal-constant-time",
    "@aws-sdk/is-array-buffer",
    "type",
    "acorn-jsx",
    "@aws-sdk/protocol-http",
    "natural-compare",
    "babylon",
    "@isaacs/cliui",
    "map-obj",
    "object-is",
    "ext",
    "esniff",
    "jackspeak",
    "watchpack",
    "unique-filename",
    "doctrine",
    "md5.js",
    "create-hash",
    "serialize-javascript",
    "@babel/helper-optimise-call-expression",
    "enhanced-resolve",
    "@babel/helper-member-expression-to-functions",
    "@smithy/service-error-classification",
    "@babel/helper-replace-supers",
    "querystringify",
    "clean-stack",
    "query-string",
    "camel-case",
    "strict-uri-encode",
    "@babel/helper-skip-transparent-expression-wrappers",
    "jwa",
    "@aws-crypto/ie11-detection",
    "@smithy/signature-v4",
    "package-json-from-dist",
    "@types/parse-json",
    "@smithy/core",
    "url-parse",
    "cacache",
    "@babel/helper-create-class-features-plugin",
    "@types/estree",
    "terser",
    "path-dirname",
    "xml2js",
    "@babel/plugin-transform-modules-commonjs",
    "camelcase-keys",
    "prr",
    "lodash.merge",
    "whatwg-encoding",
    "@aws-sdk/credential-provider-env",
    "lodash.uniq",
    "@tootallnate/once",
    "meow",
    "regenerate-unicode-properties",
    "from2",
    "@aws-sdk/credential-provider-web-identity",
    "webpack-sources",
    "@pkgjs/parseargs",
    "open",
    "@aws-sdk/credential-provider-sso",
    "nice-try",
    "@smithy/util-retry",
    "jws",
    "unicode-property-aliases-ecmascript",
    "unicode-canonical-property-names-ecmascript",
    "create-hmac",
    "babel-code-frame",
    "url-parse-lax",
    "unicode-match-property-value-ecmascript",
    "unicode-match-property-ecmascript",
    "core-js-compat",
    "errno",
    "dot-prop",
    "compressible",
    "detect-libc",
    "@babel/plugin-transform-parameters",
    "glob-to-regexp",
    "param-case",
    "@aws-sdk/credential-provider-process",
    "espree",
    "jest-worker",
    "has",
    "json-stable-stringify-without-jsonify",
    "timed-out",
    "object.values",
    "browserify-aes",
    "@babel/plugin-transform-destructuring",
    "@babel/plugin-transform-arrow-functions",
    "upath",
    "@babel/plugin-transform-classes",
    "babel-types",
    "slice-ansi",
    "@babel/plugin-transform-shorthand-properties",
    "pretty-format",
    "@babel/plugin-transform-block-scoping",
    "node-forge",
    "@aws-sdk/token-providers",
    "@babel/plugin-transform-spread",
    "@babel/plugin-transform-computed-properties",
    "jsonify",
    "@babel/plugin-transform-function-name",
    "@babel/plugin-transform-literals",
    "promise-inflight",
    "@babel/helper-create-regexp-features-plugin",
    "kleur",
    "is-retry-allowed",
    "event-target-shim",
    "for-own",
    "ejs",
    "@babel/helper-wrap-function",
    "@babel/plugin-transform-template-literals",
    "whatwg-mimetype",
    "buffer-xor",
    "leven",
    "@babel/plugin-transform-unicode-regex",
    "@babel/helper-define-polyfill-provider",
    "smart-buffer",
    "@babel/plugin-transform-for-of",
    "@babel/helper-remap-async-to-generator",
    "node-addon-api",
    "@babel/plugin-transform-sticky-regex",
    "@babel/plugin-syntax-import-attributes",
    "evp_bytestokey",
    "babel-plugin-polyfill-corejs3",
    "babel-plugin-polyfill-regenerator",
    "babel-plugin-polyfill-corejs2",
    "global-prefix",
    "@babel/plugin-transform-block-scoped-functions",
    "@smithy/config-resolver",
    "@smithy/credential-provider-imds",
    "@babel/plugin-transform-object-super",
    "@babel/plugin-transform-async-to-generator",
    "clean-css",
    "webpack",
    "invert-kv",
    "color-support",
    "rechoir",
    "lodash.memoize",
    "@babel/plugin-transform-property-literals",
    "@babel/plugin-transform-member-expression-literals",
    "boom",
    "@babel/plugin-transform-exponentiation-operator",
    "async-each",
    "@babel/plugin-transform-regenerator",
    "global-modules",
    "@smithy/middleware-retry",
    "@aws-sdk/property-provider",
    "@aws-sdk/util-uri-escape",
    "source-list-map",
    "loader-runner",
    "socks",
    "esquery",
    "@smithy/util-endpoints",
    "progress",
    "lcid",
    "@babel/plugin-transform-named-capturing-groups-regex",
    "typedarray-to-buffer",
    "duplexer3",
    "osenv",
    "pbkdf2",
    "duplexer",
    "aggregate-error",
    "@aws-sdk/util-buffer-from",
    "abort-controller",
    "trim-newlines",
    "@aws-sdk/credential-provider-ini",
    "compression",
    "vue",
    "@types/qs",
    "flat-cache",
    "@babel/plugin-transform-dotall-regex",
    "babel-plugin-macros",
    "@babel/plugin-transform-typeof-symbol",
    "@babel/plugin-transform-modules-amd",
    "@babel/plugin-transform-duplicate-keys",
    "@babel/plugin-transform-modules-systemjs",
    "@babel/plugin-transform-modules-umd",
    "@babel/plugin-transform-new-target",
    "xml-name-validator",
    "path-browserify",
    "acorn-globals",
    "@babel/preset-env",
    "pirates",
    "@smithy/hash-node",
    "@smithy/invalid-dependency",
    "@smithy/middleware-content-length",
    "@smithy/util-body-length-node",
    "@smithy/util-defaults-mode-browser",
    "@smithy/util-defaults-mode-node",
    "array-includes",
    "csso",
    "os-locale",
    "@babel/plugin-transform-reserved-words",
    "shell-quote",
    "node-int64",
    "file-entry-cache",
    "@babel/preset-modules",
    "object-hash",
    "@inquirer/figures",
    "path-is-inside",
    "@babel/plugin-transform-optional-chaining",
    "normalize-range",
    "stack-trace",
    "yoctocolors-cjs",
    "@types/yargs",
    "cssstyle",
    "@aws-sdk/core",
    "@babel/plugin-transform-unicode-escapes",
    "crypto-random-string",
    "dot-case",
    "mdn-data",
    "@types/yargs-parser",
    "@babel/plugin-transform-private-methods",
    "sisteransi",
    "autoprefixer",
    "svgo",
    "@vue/compiler-sfc",
    "css-tree",
    "ansi-colors",
    "@babel/plugin-transform-private-property-in-object",
    "websocket-extensions",
    "@babel/plugin-syntax-import-assertions",
    "type-detect",
    "@babel/plugin-proposal-private-property-in-object",
    "@babel/plugin-transform-nullish-coalescing-operator",
    "@babel/plugin-transform-class-properties",
    "@babel/plugin-bugfix-v8-spread-parameters-in-optional-chaining",
    "@babel/plugin-bugfix-safari-id-destructuring-collision-in-function-expression",
    "@types/connect",
    "jsdom",
    "@babel/plugin-syntax-typescript",
    "symbol-tree",
    "@babel/plugin-transform-object-rest-spread",
    "cssom",
    "@babel/plugin-syntax-object-rest-spread",
    "@babel/plugin-transform-export-namespace-from",
    "dom-helpers",
    "@babel/plugin-transform-async-generator-functions",
    "@babel/plugin-transform-numeric-separator",
    "@babel/plugin-transform-optional-catch-binding",
    "@babel/plugin-transform-logical-assignment-operators",
    "faye-websocket",
    "websocket-driver",
    "flatted",
    "@babel/plugin-transform-class-static-block",
    "@babel/plugin-transform-dynamic-import",
    "@babel/plugin-transform-unicode-property-regex",
    "@babel/plugin-transform-json-strings",
    "@babel/plugin-syntax-unicode-sets-regex",
    "@babel/plugin-transform-unicode-sets-regex",
    "resolve-cwd",
    "magic-string",
    "any-promise",
    "unique-string",
    "postcss-modules-local-by-default",
    "postcss-modules-scope",
    "@emotion/memoize",
    "jest-util",
    "lazy-cache",
    "@babel/plugin-bugfix-v8-static-class-fields-redefine-readonly",
    "postcss-modules-extract-imports",
    "@babel/plugin-bugfix-firefox-class-in-computed-class-key",
    "@types/istanbul-lib-coverage",
    "@babel/plugin-bugfix-safari-class-field-initializer-scope",
    "@babel/plugin-transform-duplicate-named-capturing-groups-regex",
    "querystring",
    "@babel/plugin-transform-regexp-modifiers",
    "postcss-modules-values",
    "pretty-bytes",
    "p-cancelable",
    "eslint",
    "@types/react",
    "humanize-ms",
    "pumpify",
    "@xtuc/long",
    "pend",
    "jsonparse",
    "@webassemblyjs/floating-point-hex-parser",
    "bignumber.js",
    "@webassemblyjs/helper-api-error",
    "@webassemblyjs/helper-wasm-bytecode",
    "abab",
    "@webassemblyjs/ast",
    "bser",
    "object.getownpropertydescriptors",
    "fb-watchman",
    "pascal-case",
    "@types/istanbul-lib-report",
    "@webassemblyjs/wast-printer",
    "@babel/plugin-transform-react-jsx",
    "es-shim-unscopables",
    "chrome-trace-event",
    "http-proxy",
    "lodash.isstring",
    "prompts",
    "@xtuc/ieee754",
    "http-parser-js",
    "@types/istanbul-reports",
    "@webassemblyjs/leb128",
    "@webassemblyjs/ieee754",
    "@webassemblyjs/utf8",
    "json-stable-stringify",
    "asn1.js",
    "es-array-method-boxes-properly",
    "terser-webpack-plugin",
    "object.fromentries",
    "@webassemblyjs/wasm-gen",
    "@webassemblyjs/helper-buffer",
    "@webassemblyjs/wasm-parser",
    "import-local",
    "babel-messages",
    "arg",
    "@jest/types",
    "@aws-sdk/util-hex-encoding",
    "assert",
    "fstream",
    "@webassemblyjs/helper-wasm-section",
    "@webassemblyjs/wasm-opt",
    "agentkeepalive",
    "@webassemblyjs/wasm-edit",
    "stable",
    "@aws-sdk/region-config-resolver",
    "tmpl",
    "@emotion/unitless",
    "iferr",
    "eslint-import-resolver-node",
    "detect-newline",
    "istanbul-lib-coverage",
    "relateurl",
    "react-transition-group",
    "estree-walker",
    "dayjs",
    "detect-indent",
    "makeerror",
    "builtin-modules",
    "astral-regex",
    "fd-slicer",
    "html-entities",
    "ip",
    "walker",
    "array.prototype.flat",
    "shelljs",
    "eslint-module-utils",
    "@aws-sdk/credential-provider-node",
    "node-gyp-build",
    "icss-utils",
    "window-size",
    "@babel/plugin-proposal-class-properties",
    "socks-proxy-agent",
    "yauzl",
    "babel-traverse",
    "eslint-plugin-import",
    "array.prototype.reduce",
    "@types/mime",
    "html-encoding-sniffer",
    "@babel/plugin-transform-react-display-name",
    "jest-diff",
    "jest-get-type",
    "stack-utils",
    "jsonpointer",
    "object.entries",
    "@typescript-eslint/typescript-estree",
    "babel-loader",
    "long",
    "postcss-load-config",
    "@babel/plugin-transform-typescript",
    "@babel/plugin-syntax-optional-chaining",
    "caniuse-api",
    "cryptiles",
    "infer-owner",
    "zod",
    "@noble/hashes",
    "batch",
    "detect-node",
    "crypto-browserify",
    "@types/send",
    "css-loader",
    "err-code",
    "async-limiter",
    "@types/range-parser",
    "jest-mock",
    "@types/http-errors",
    "postcss-calc",
    "test-exclude",
    "jest-message-util",
    "@babel/plugin-syntax-nullish-coalescing-operator",
    "stream-browserify",
    "@types/minimatch",
    "postcss-discard-duplicates",
    "@types/express-serve-static-core",
    "postcss-discard-empty",
    "jsx-ast-utils",
    "postcss-merge-rules",
    "postcss-discard-comments",
    "postcss-minify-selectors",
    "@sindresorhus/is",
    "browserify-zlib",
    "postcss-merge-longhand",
    "postcss-ordered-values",
    "postcss-convert-values",
    "postcss-unique-selectors",
    "postcss-colormin",
    "postcss-normalize-url",
    "@types/body-parser",
    "lodash._reinterpolate",
    "@types/serve-static",
    "postcss-minify-gradients",
    "postcss-minify-font-values",
    "postcss-minify-params",
    "postcss-svgo",
    "postcss-reduce-transforms",
    "postcss-normalize-charset",
    "quick-lru",
    "postcss-discard-overridden",
    "postcss-reduce-initial",
    "istanbul-lib-instrument",
    "resize-observer-polyfill",
    "sntp",
    "raf",
    "buffer-alloc-unsafe",
    "dateformat",
    "@types/express",
    "buffer-fill",
    "jest-haste-map",
    "responselike",
    "cssnano",
    "private",
    "ua-parser-js",
    "lodash.isboolean",
    "console-browserify",
    "@babel/plugin-transform-runtime",
    "buffer-alloc",
    "sort-keys",
    "jest-regex-util",
    "lodash.templatesettings",
    "jest-matcher-utils",
    "coa",
    "min-indent",
    "eslint-plugin-react",
    "parse-asn1",
    "browserify-rsa",
    "serve-index",
    "js-base64",
    "is-ci",
    "util.promisify",
    "ajv-formats",
    "des.js",
    "replace-ext",
    "clsx",
    "lodash.once",
    "node-gyp",
    "lodash.template",
    "cacheable-request",
    "safe-stable-stringify",
    "array.prototype.flatmap",
    "utila",
    "immer",
    "@aws-sdk/shared-ini-file-loader",
    "aria-query",
    "tsutils",
    "@jridgewell/source-map",
    "ip-address",
    "lodash.isarguments",
    "@babel/plugin-syntax-optional-catch-binding",
    "style-loader",
    "webpack-dev-middleware",
    "thenify",
    "miller-rabin",
    "cors",
    "flush-write-stream",
    "duplexer2",
    "exit",
    "querystring-es3",
    "os-browserify",
    "dom-converter",
    "randomfill",
    "@types/normalize-package-data",
    "@typescript-eslint/parser",
    "https-browserify",
    "vm-browserify",
    "jsonwebtoken",
    "tsconfig-paths",
    "lodash.isnumber",
    "stylis",
    "constants-browserify",
    "domain-browser",
    "lodash.includes",
    "renderkid",
    "babel-plugin-istanbul",
    "jest-validate",
    "tty-browserify",
    "timers-browserify",
    "array-find-index",
    "istanbul-lib-report",
    "babel-template",
    "browserify-des",
    "istanbul-reports",
    "dns-packet",
    "diffie-hellman",
    "create-ecdh",
    "public-encrypt",
    "browserify-sign",
    "pretty-error",
    "data-urls",
    "lodash.isinteger",
    "widest-line",
    "p-timeout",
    "@babel/preset-react",
    "browserify-cipher",
    "clone-response",
    "http-proxy-middleware",
    "clone-stats",
    "nwsapi",
    "is-promise",
    "thenify-all",
    "validate-npm-package-name",
    "@babel/preset-typescript",
    "@babel/plugin-transform-react-jsx-development",
    "filelist",
    "builtin-status-codes",
    "string-length",
    "jake",
    "connect-history-api-fallback",
    "mz",
    "@types/stack-utils",
    "randomatic",
    "@babel/plugin-syntax-async-generators",
    "@typescript-eslint/eslint-plugin",
    "winston",
    "decimal.js",
    "stream-http",
    "babel-plugin-jest-hoist",
    "istanbul-lib-source-maps",
    "env-paths",
    "diff-sequences",
    "pkg-up",
    "@babel/plugin-transform-react-pure-annotations",
    "isomorphic-fetch",
    "jest-environment-node",
    "rollup",
    "babel-preset-jest",
    "sigmund",
    "is-primitive",
    "colorette",
    "vinyl",
    "throat",
    "thunky",
    "domexception",
    "cli-boxes",
    "file-loader",
    "babel-jest",
    "@types/json5",
    "xmlchars",
    "@emotion/is-prop-valid",
    "@aws-sdk/util-middleware",
    "cyclist",
    "longest",
    "@babel/plugin-syntax-numeric-separator",
    "string.prototype.matchall",
    "shallow-clone",
    "loud-rejection",
    "minipass-collect",
    "minipass-flush",
    "minipass-pipeline",
    "@typescript-eslint/types",
    "es6-promisify",
    "@types/glob",
    "lodash.defaults",
    "clone-deep",
    "jest-snapshot",
    "saxes",
    "@typescript-eslint/visitor-keys",
    "currently-unhandled",
    "lodash.camelcase",
    "fs-write-stream-atomic",
    "filesize",
    "jest-resolve",
    "obuf",
    "postcss-loader",
    "gzip-size",
    "expect",
    "split2",
    "handlebars",
    "symbol-observable",
    "html-webpack-plugin",
    "file-type",
    "hawk",
    "multicast-dns",
    "@babel/plugin-syntax-json-strings",
    "wbuf",
    "html-escaper",
    "table",
    "proto-list",
    "parallel-transform",
    "jest-docblock",
    "object.omit",
    "spdy",
    "stream-each",
    "hpack.js",
    "lodash-es",
    "handle-thing",
    "select-hose",
    "spdy-transport",
    "http-deceiver",
    "ip-regex",
    "address",
    "axobject-query",
    "@emotion/hash",
    "react-lifecycles-compat",
    "to-arraybuffer",
    "@aws-sdk/querystring-parser",
    "promise-retry",
    "glob-base",
    "browser-process-hrtime",
    "expand-range",
    "@typescript-eslint/scope-manager",
    "@types/babel__traverse",
    "sockjs",
    "jest-config",
    "jest-runtime",
    "damerau-levenshtein",
    "@aws-sdk/signature-v4",
    "shallowequal",
    "is-dotfile",
    "is-equal-shallow",
    "@colors/colors",
    "stackframe",
    "stylehacks",
    "selfsigned",
    "preserve",
    "npm-normalize-package-bin",
    "memory-fs",
    "lodash.sortby",
    "@types/babel__generator",
    "error-stack-parser",
    "jest-changed-files",
    "eslint-plugin-jsx-a11y",
    "capture-stack-trace",
    "@types/babel__template",
    "ast-types-flow",
    "postcss-normalize-whitespace",
    "@types/babel__core",
    "lodash.clonedeep",
    "jest-resolve-dependencies",
    "css-declaration-sorter",
    "postcss-normalize-positions",
    "postcss-normalize-display-values",
    "postcss-normalize-repeat-style",
    "postcss-normalize-string",
    "@babel/plugin-proposal-optional-chaining",
    "cross-fetch",
    "postcss-normalize-timing-functions",
    "postcss-normalize-unicode",
    "w3c-xmlserializer",
    "mississippi",
    "w3c-hr-time",
    "@jest/fake-timers",
    "math-random",
    "@babel/plugin-syntax-flow",
    "warning",
    "core-js-pure",
    "cssnano-preset-default",
    "is-posix-bracket",
    "@sinonjs/commons",
    "@babel/plugin-transform-flow-strip-types",
    "@aws-sdk/querystring-builder",
    "block-stream",
    "jest-cli",
    "webpack-dev-server",
    "config-chain",
    "parse-glob",
    "object.groupby",
    "lodash._basetostring",
    "array.prototype.findlastindex",
    "run-queue",
    "filename-regex",
    "regex-cache",
    "get-package-type",
    "supports-hyperlinks",
    "unzip-response",
    "date-fns",
    "define-lazy-prop",
    "is-redirect",
    "array-differ",
    "@rtsao/scc",
    "@babel/plugin-syntax-class-properties",
    "builtins",
    "caller-path",
    "@babel/plugin-proposal-nullish-coalescing-operator",
    "is-property",
    "create-error-class",
    "babel-core",
    "@aws-sdk/node-config-provider",
    "@types/ws",
    "cheerio",
    "@types/prop-types",
    "is-path-cwd",
    "jest",
    "@babel/plugin-syntax-decorators",
    "copy-concurrently",
    "xdg-basedir",
    "npm-package-arg",
    "@jest/environment",
    "@babel/plugin-proposal-decorators",
    "dotenv-expand",
    "jest-serializer",
    "@aws-sdk/abort-controller",
    "unquote",
    "lodash._getnative",
    "sass-loader",
    "@sinclair/typebox",
    "@jest/transform",
    "is-generator-fn",
    "jest-environment-jsdom",
    "fbjs",
    "@npmcli/fs",
    "crc-32",
    "lodash.keys",
    "@aws-sdk/url-parser",
    "@types/q",
    "optimist",
    "move-concurrently",
    "p-retry",
    "del",
    "eslint-plugin-react-hooks",
    "simple-concat",
    "jest-runner",
    "@jest/console",
    "upper-case",
    "jest-leak-detector",
    "@jest/test-result",
    "@babel/plugin-syntax-top-level-await",
    "validator",
    "tiny-warning",
    "recursive-readdir",
    "@jest/source-map",
    "is-root",
    "css-select-base-adapter",
    "node-libs-browser",
    "jest-jasmine2",
    "jest-each",
    "mini-css-extract-plugin",
    "jest-pnp-resolver",
    "@rollup/pluginutils",
    "jest-watcher",
    "p-is-promise",
    "make-fetch-happen",
    "common-tags",
    "ansi-html",
    "default-gateway",
    "trim-right",
    "align-text",
    "node-uuid",
    "lazystream",
    "confusing-browser-globals",
    "string.prototype.repeat",
    "array.prototype.tosorted",
    "tar-fs",
    "@eslint-community/eslint-utils",
    "iterator.prototype",
    "es-iterator-helpers",
    "lodash.isarray",
    "@eslint/eslintrc",
    "ast-types",
    "@babel/plugin-syntax-private-property-in-object",
    "array.prototype.findlast",
    "tiny-invariant",
    "generate-function",
    "reflect-metadata",
    "dedent",
    "@aws-crypto/crc32",
    "@aws-sdk/service-error-classification",
    "user-home",
    "is-regexp",
    "uniq",
    "registry-url",
    "@jest/test-sequencer",
    "@aws-sdk/util-config-provider",
    "react-error-overlay",
    "ansi-align",
    "@jest/reporters",
    "mem",
    "worker-farm",
    "registry-auth-token",
    "is-relative",
    "base-x",
    "@istanbuljs/schema",
    "detect-port-alt",
    "@protobufjs/inquire",
    "@protobufjs/aspromise",
    "simple-get",
    "react-dev-utils",
    "@jest/core",
    "sourcemap-codec",
    "@sinonjs/fake-timers",
    "functional-red-black-tree",
    "boxen",
    "stringstream",
    "@types/eslint",
    "@aws-sdk/config-resolver",
    "figgy-pudding",
    "find-root",
    "right-align",
    "@jest/schemas",
    "make-error",
    "@babel/plugin-syntax-logical-assignment-operators",
    "postcss-flexbugs-fixes",
    "fast-safe-stringify",
    "center-align",
    "is-directory",
    "@hapi/hoek",
    "@aws-sdk/middleware-stack",
    "fecha",
    "@eslint-community/regexpp",
    "@humanwhocodes/object-schema",
    "lilconfig",
    "@protobufjs/utf8",
    "@protobufjs/float",
    "time-stamp",
    "@protobufjs/codegen",
    "@protobufjs/base64",
    "@protobufjs/eventemitter",
    "@protobufjs/pool",
    "@protobufjs/path",
    "@protobufjs/fetch",
    "@types/semver",
    "eventsource",
    "react-refresh",
    "graphemer",
    "@types/graceful-fs",
    "defer-to-connect",
    "lodash.get",
    "case-sensitive-paths-webpack-plugin",
    "defined",
    "@humanwhocodes/config-array",
    "@babel/plugin-syntax-import-meta",
    "configstore",
    "fork-ts-checker-webpack-plugin",
    "is-absolute",
    "@istanbuljs/load-nyc-config",
    "lodash.isequal",
    "eslint-plugin-flowtype",
    "data-uri-to-buffer",
    "@aws-sdk/middleware-serde",
    "immutable",
    "@babel/plugin-proposal-numeric-separator",
    "axe-core",
    "html-minifier-terser",
    "home-or-tmp",
    "postcss-nesting",
    "JSONStream",
    "is-potential-custom-element-name",
    "opn",
    "postcss-custom-properties",
    "uglify-to-browserify",
    "postcss-custom-media",
    "stringify-object",
    "bs58",
    "@aws-sdk/credential-provider-imds",
    "postcss-selector-not",
    "@aws-sdk/node-http-handler",
    "watchpack-chokidar2",
    "postcss-custom-selectors",
    "@aws-sdk/client-sts",
    "@types/lodash",
    "@szmarczak/http-timer",
    "babel-plugin-transform-react-remove-prop-types",
    "postcss-color-hex-alpha",
    "temp-dir",
    "postcss-font-variant",
    "postcss-pseudo-class-any-link",
    "postcss-color-rebeccapurple",
    "call-me-maybe",
    "eslint-config-react-app",
    "lodash._root",
    "postcss-replace-overflow-wrap",
    "ajv-errors",
    "postcss-attribute-case-insensitive",
    "protobufjs",
    "@babel/plugin-syntax-bigint",
    "@typescript-eslint/experimental-utils",
    "package-json",
    "parse-passwd",
    "@aws-sdk/fetch-http-handler",
    "ansi-wrap",
    "cjs-module-lexer",
    "num2fraction",
    "@noble/curves",
    "@emotion/utils",
    "@typescript-eslint/utils",
    "homedir-polyfill",
    "@babel/plugin-syntax-class-static-block",
    "triple-beam",
    "@types/trusted-types",
    "get-own-enumerable-property-symbols",
    "is-absolute-url",
    "jiti",
    "indexes-of",
    "postcss-color-functional-notation",
    "language-subtag-registry",
    "postcss-page-break",
    "postcss-focus-visible",
    "postcss-logical",
    "postcss-dir-pseudo-class",
    "cssdb",
    "js-sha3",
    "language-tags",
    "is-path-in-cwd",
    "postcss-image-set-function",
    "postcss-overflow-shorthand",
    "postcss-place",
    "postcss-gap-properties",
    "postcss-preset-env",
    "postcss-focus-within",
    "postcss-lab-function",
    "babel-preset-current-node-syntax",
    "pluralize",
    "webpack-manifest-plugin",
    "@svgr/core",
    "@types/hyper-function-component",
    "postcss-initial",
    "crypt",
    "charenc",
    "postcss-media-minmax",
    "regexpp",
    "@aws-sdk/smithy-client",
    "@aws-sdk/hash-node",
    "babel-preset-react-app",
    "postcss-double-position-gradients",
    "@aws-sdk/middleware-retry",
    "@aws-sdk/middleware-content-length",
    "@aws-sdk/util-body-length-node",
    "ignore-walk",
    "@aws-sdk/util-body-length-browser",
    "@aws-sdk/invalid-dependency",
    "babel-generator",
    "text-hex",
    "is-object",
    "es6-weak-map",
    "string.prototype.includes",
    "es-module-lexer",
    "undici",
    "rsvp",
    "eyes",
    "@emotion/serialize",
    "svg-parser",
    "emittery",
    "engine.io-parser",
    "css-prefers-color-scheme",
    "css-blank-pseudo",
    "css-has-pseudo",
    "@babel/plugin-transform-react-constant-elements",
    "@svgr/babel-plugin-add-jsx-attribute",
    "@svgr/babel-plugin-replace-jsx-attribute-value",
    "@svgr/babel-plugin-svg-dynamic-title",
    "@svgr/babel-plugin-svg-em-dimensions",
    "@svgr/babel-plugin-transform-react-native-svg",
    "@svgr/babel-plugin-transform-svg-component",
    "@svgr/babel-plugin-remove-jsx-attribute",
    "@svgr/babel-plugin-remove-jsx-empty-expression",
    "char-regex",
    "@aws-sdk/util-utf8",
    "@svgr/babel-preset",
    "@svgr/hast-util-to-babel-ast",
    "@aws-sdk/credential-provider-http",
    "@svgr/plugin-jsx",
    "lodash.flatten",
    "@bcoe/v8-coverage",
    "v8-to-istanbul",
    "klona",
    "redux",
    "parse-node-version",
    "@types/triple-beam",
    "postcss-env-function",
    "generate-object-property",
    "is-module",
    "expand-tilde",
    "eslint-utils",
    "caller-callsite",
    "bson",
    "logform",
    "@typescript-eslint/type-utils",
    "kuler",
    "@emotion/weak-memoize",
    "@types/resolve",
    "enabled",
    "colorspace",
    "@svgr/plugin-svgo",
    "tinycolor2",
    "mkdirp-classic",
    "@emotion/sheet",
    "terminal-link",
    "collect-v8-coverage",
    "@jest/globals",
    "@hapi/topo",
    "@emotion/cache",
    "minipass-sized",
    "lodash.restparam",
    "npm-bundled",
    "harmony-reflect",
    "strip-comments",
    "web-streams-polyfill",
    "socket.io-parser",
    "loglevel",
    "workbox-core",
    "css",
    "@svgr/webpack",
    "async-validator",
    "@types/html-minifier-terser",
    "lodash._basecopy",
    "identity-obj-proxy",
    "archy",
    "workbox-routing",
    "workbox-strategies",
    "workbox-precaching",
    "react-router",
    "workbox-cacheable-response",
    "workbox-background-sync",
    "workbox-sw",
    "workbox-build",
    "fs-monkey",
    "workbox-range-requests",
    "workbox-google-analytics",
    "minipass-fetch",
    "@aws-sdk/middleware-signing",
    "findup-sync",
    "winston-transport",
    "workbox-navigation-preload",
    "workbox-streams",
    "import-lazy",
    "resolve-dir",
    "babel-helpers",
    "camelize",
    "babel-register",
    "read",
    "ansi-html-community",
    "strip-dirs",
    "@webassemblyjs/helper-numbers",
    "babel-helper-get-function-arity",
    "one-time",
    "@types/prettier",
    "css-color-names",
    "@types/ms",
    "fraction.js",
    "@aws-sdk/client-sso-oidc",
    "latest-version",
    "fn.name",
    "@types/react-dom",
    "p-defer",
    "static-eval",
    "regex-parser",
    "babel-plugin-named-asset-import",
    "workbox-webpack-plugin",
    "@babel/plugin-proposal-private-methods",
    "react-app-polyfill",
    "unbzip2-stream",
    "traverse",
    "throttle-debounce",
    "ansi-gray",
    "denque",
    "dom-walk",
    "min-document",
    "@types/eslint-scope",
    "stealthy-require",
    "indexof",
    "request-promise-core",
    "is-resolvable",
    "resolve-url-loader",
    "adjust-sourcemap-loader",
    "@types/react-transition-group",
    "use-sync-external-store",
    "@types/unist",
    "mongodb",
    "is-natural-number",
    "global",
    "lodash._isiterateecall",
    "es-get-iterator",
    "lodash.escape",
    "@rollup/plugin-node-resolve",
    "jest-watch-typeahead",
    "workbox-expiration",
    "@dabh/diagnostics",
    "babel-helper-function-name",
    "workbox-window",
    "workbox-broadcast-update",
    "uniqs",
    "node-domexception",
    "alphanum-sort",
    "filename-reserved-regex",
    "formidable",
    "npm-packlist",
    "fancy-log",
    "@ungap/structured-clone",
    "@eslint/js",
    "jest-circus",
    "@popperjs/core",
    "@csstools/normalize.css",
    "first-chunk-stream",
    "jmespath",
    "postcss-normalize",
    "postcss-browser-comments",
    "has-symbol-support-x",
    "camelcase-css",
    "filenamify",
    "check-types",
    "has-to-string-tag-x",
    "flatten",
    "seek-bzip",
    "@babel/highlight",
    "babel-plugin-syntax-object-rest-spread",
    "strip-outer",
    "@trysound/sax",
    "trim-repeated",
    "react-scripts",
    "decompress-tar",
    "xmlhttprequest-ssl",
    "@types/retry",
    "@npmcli/move-file",
    "eslint-plugin-jest",
    "@webassemblyjs/helper-code-frame",
    "@webassemblyjs/helper-fsm",
    "@webassemblyjs/wast-parser",
    "vendors",
    "@gar/promisify",
    "yn",
    "decompress",
    "@webassemblyjs/helper-module-context",
    "is-my-json-valid",
    "tempy",
    "exec-sh",
    "socket.io-client",
    "decompress-targz",
    "decompress-unzip",
    "decompress-tarbz2",
    "semver-diff",
    "portfinder",
    "ultron",
    "marked",
    "isurl",
    "url-to-options",
    "@humanwhocodes/module-importer",
    "tryer",
    "crypto-js",
    "image-size",
    "css.escape",
    "engine.io-client",
    "immediate",
    "ts-interface-checker",
    "import-from",
    "sucrase",
    "array-equal",
    "css-color-keywords",
    "jquery",
    "colord",
    "@aws-sdk/util-defaults-mode-browser",
    "@aws-sdk/util-defaults-mode-node",
    "@types/jest",
    "read-cache",
    "sanitize.css",
    "@ethersproject/logger",
    "sparkles",
    "react-router-dom",
    "is-npm",
    "hoopy",
    "@types/uuid",
    "bfj",
    "write",
    "big-integer",
    "globule",
    "ts-node",
    "@ethersproject/bytes",
    "@types/http-proxy",
    "crc32-stream",
    "@babel/plugin-proposal-object-rest-spread",
    "graphql",
    "md5",
    "underscore.string",
    "postcss-import",
    "css-to-react-native",
    "@pmmmwh/react-refresh-webpack-plugin",
    "memfs",
    "url-loader",
    "path-root-regex",
    "download",
    "shellwords",
    "rollup-plugin-terser",
    "sane",
    "memoize-one",
    "rx-lite",
    "js-cookie",
    "base64-arraybuffer",
    "babel-plugin-transform-strict-mode",
    "babel-plugin-transform-object-rest-spread",
    "path-root",
    "postcss-js",
    "babel-helper-hoist-variables",
    "update-notifier",
    "tsscmp",
    "glogg",
    "minimist-options",
    "postcss-nested",
    "decamelize-keys",
    "graceful-readlink",
    "growly",
    "unc-path-regex",
    "v8-compile-cache",
    "term-size",
    "babel-plugin-syntax-trailing-function-commas",
    "jsonschema",
    "gaze",
    "multipipe",
    "@rollup/plugin-replace",
    "parse5-htmlparser2-tree-adapter",
    "slide",
    "@aws-sdk/util-base64",
    "dom-accessibility-api",
    "cssnano-utils",
    "babel-plugin-transform-es2015-modules-commonjs",
    "is-unc-path",
    "sockjs-client",
    "@leichtgewicht/ip-codec",
    "node-notifier",
    "@ethersproject/bignumber",
    "global-dirs",
    "gulplog",
    "babel-helper-optimise-call-expression",
    "natural-compare-lite",
    "jsonpath",
    "joi",
    "dlv",
    "tailwindcss",
    "resolve.exports",
    "klaw",
    "regenerator-transform",
    "toggle-selection",
    "@mrmlnc/readdir-enhanced",
    "didyoumean",
    "unist-util-stringify-position",
    "string-natural-compare",
    "clone-buffer",
    "copy-to-clipboard",
    "cacheable-lookup",
    "babel-helper-replace-supers",
    "@testing-library/react",
    "@testing-library/dom",
    "styled-components",
    "babel-helper-define-map",
    "idb",
    "ncp",
    "is-lambda",
    "create-require",
    "babel-helper-regex",
    "babel-helper-call-delegate",
    "character-entities",
    "@types/long",
    "github-from-package",
    "expand-template",
    "json3",
    "node-abi",
    "eslint-webpack-plugin",
    "@floating-ui/core",
    "@emotion/babel-plugin",
    "unist-util-is",
    "lz-string",
    "@babel/plugin-syntax-dynamic-import",
    "@types/http-cache-semantics",
    "babel-plugin-syntax-jsx",
    "dezalgo",
    "popper.js",
    "babel-plugin-transform-es2015-destructuring",
    "browser-resolve",
    "@types/debug",
    "unique-stream",
    "ordered-read-streams",
    "babel-plugin-transform-es2015-block-scoping",
    "@ethersproject/keccak256",
    "@floating-ui/utils",
    "memory-pager",
    "recast",
    "babel-plugin-transform-es2015-parameters",
    "babel-plugin-transform-es2015-spread",
    "babel-plugin-transform-es2015-for-of",
    "@floating-ui/dom",
    "@testing-library/jest-dom",
    "cloneable-readable",
    "babel-plugin-transform-es2015-template-literals",
    "babel-plugin-transform-es2015-function-name",
    "@ethersproject/rlp",
    "lodash._basevalues",
    "babel-plugin-check-es2015-constants",
    "babel-plugin-transform-es2015-shorthand-properties",
    "babel-plugin-transform-es2015-arrow-functions",
    "eslint-plugin-testing-library",
    "glob-stream",
    "sparse-bitfield",
    "npm-registry-fetch",
    "babel-plugin-transform-es2015-computed-properties",
    "babel-plugin-transform-es2015-classes",
    "babel-eslint",
    "@ethersproject/address",
    "babel-plugin-transform-es2015-literals",
    "@surma/rollup-plugin-off-main-thread",
    "scrypt-js",
    "compress-commons",
    "babel-plugin-transform-es2015-unicode-regex",
    "babel-plugin-transform-es2015-sticky-regex",
    "rc-util",
    "gulp-util",
    "aes-js",
    "ctype",
    "@ethersproject/constants",
    "babel-plugin-transform-regenerator",
    "bufferutil",
    "@testing-library/user-event",
    "npm-install-checks",
    "split-on-first",
    "babel-plugin-transform-es2015-block-scoped-functions",
    "babel-plugin-transform-es2015-object-super",
    "internal-ip",
    "get-proxy",
    "prebuild-install",
    "map-age-cleaner",
    "@aws-sdk/util-utf8-node",
    "dequal",
    "buffer-indexof",
    "cli-table",
    "detect-file",
    "@scure/base",
    "utf-8-validate",
    "@hapi/bourne",
    "beeper",
    "cookies",
    "keygrip",
    "is-installed-globally",
    "filter-obj",
    "@aws-sdk/middleware-sdk-sts",
    "bech32",
    "teanager",
    "history",
    "react-fast-compare",
    "multicast-dns-service-types",
    "v8flags",
    "import-cwd",
    "pacote",
    "moment-timezone",
    "npm-pick-manifest",
    "lodash.throttle",
    "has-gulplog",
    "@alloc/quick-lru",
    "caw",
    "cheerio-select",
    "web-vitals",
    "resolve-alpn",
    "flat",
    "lodash._reescape",
    "lodash._reevaluate",
    "base64id",
    "unist-util-visit-parents",
    "@emotion/use-insertion-effect-with-fallbacks",
    "dns-equal",
    "@ethersproject/properties",
    "archiver-utils",
    "dns-txt",
    "lodash.union",
    "@ethersproject/strings",
    "@types/aria-query",
    "lodash.truncate",
    "@types/node-fetch",
    "textextensions",
    "@babel/eslint-parser",
    "binaryextensions",
    "babel-plugin-transform-es2015-typeof-symbol",
    "acertea",
    "rylie",
    "@ethersproject/signing-key",
    "untildify",
    "d3-color",
    "@ethersproject/transactions",
    "babel-plugin-transform-es2015-modules-amd",
    "@adobe/css-tools",
    "@nicolo-ribaudo/eslint-scope-5-internals",
    "extract-zip",
    "babel-plugin-transform-es2015-duplicate-keys",
    "anakjalanan",
    "babel-plugin-transform-es2015-modules-systemjs",
    "@jest/expect-utils",
    "@types/node-forge",
    "bonjour",
    "capture-exit",
    "character-entities-legacy",
    "natives",
    "esbuild",
    "d3-interpolate",
    "@emotion/react",
    "http2-wrapper",
    "launch-editor",
    "multimatch",
    "@ethersproject/base64",
    "connect",
    "@ethersproject/networks",
    "babel-plugin-transform-es2015-modules-umd",
    "@ethersproject/web",
    "@aws-sdk/middleware-endpoint",
    "lie",
    "@ethersproject/abstract-provider",
    "proc-log",
    "ansicolors",
    "@azure/abort-controller",
    "lodash.difference",
    "@aws-sdk/util-retry",
    "nitroteh",
    "unist-util-visit",
    "@ethersproject/abstract-signer",
    "@tsconfig/node14",
    "@cspotcode/source-map-support",
    "@ethersproject/hash",
    "vue-router",
    "v8-compile-cache-lib",
    "@tsconfig/node16",
    "@types/minimist",
    "enquirer",
    "killable",
    "isomorphic-ws",
    "cycle",
    "napi-build-utils",
    "create-react-class",
    "strip-bom-stream",
    "archive-type",
    "array-slice",
    "ramda",
    "read-package-json",
    "@tsconfig/node12",
    "webpack-log",
    "d3-array",
    "@emotion/stylis",
    "vfile",
    "@tsconfig/node10",
    "vfile-message",
    "socket.io",
    "vinyl-fs",
    "@types/connect-history-api-fallback",
    "@esbuild/linux-loong64",
    "zip-stream",
    "@csstools/selector-specificity",
    "babel-plugin-syntax-async-functions",
    "@types/responselike",
    "exit-hook",
    "@types/keyv",
    "engine.io",
    "socket.io-adapter",
    "dargs",
    "backo2",
    "@types/testing-library__jest-dom",
    "into-stream",
    "highlight.js",
    "react-redux",
    "istextorbinary",
    "bonjour-service",
    "uid-number",
    "hyphenate-style-name",
    "is-builtin-module",
    "is-alphabetical",
    "is-decimal",
    "@types/serve-index",
    "@types/sockjs",
    "@types/bonjour",
    "is-alphanumerical",
    "@rushstack/eslint-patch",
    "detective",
    "json-bigint",
    "@rollup/plugin-babel",
    "p-event",
    "snake-case",
    "@aws-sdk/xml-builder",
    "icss-replace-symbols",
    "consola",
    "html-minifier",
    "css-minimizer-webpack-plugin",
    "@scure/bip39",
    "is-hexadecimal",
    "exponential-backoff",
    "babel-helper-explode-assignable-expression",
    "request-promise-native",
    "babel-helper-remap-async-to-generator",
    "is-function",
    "mdurl",
    "character-reference-invalid",
    "hex-color-regex",
    "@babel/runtime-corejs3",
    "basic-auth",
    "@types/cacheable-request",
    "parse-entities",
    "upper-case-first",
    "cmd-shim",
    "hsl-regex",
    "hsla-regex",
    "rgba-regex",
    "rgb-regex",
    "hard-rejection",
    "@ethersproject/abi",
    "bail",
    "@types/hoist-non-react-statics",
    "trough",
    "babel-plugin-syntax-exponentiation-operator",
    "babel-helper-builder-binary-assignment-operator-visitor",
    "fs-readdir-recursive",
    "ethers",
    "source-map-loader",
    "is-color-stop",
    "unique-names-generator",
    "babel-plugin-transform-async-to-generator",
    "aws-sdk",
    "is-my-ip-valid",
    "@csstools/postcss-progressive-custom-properties",
    "@smithy/eventstream-codec",
    "@csstools/postcss-is-pseudo-class",
    "postcss-opacity-percentage",
    "postcss-clamp",
    "timsort",
    "@csstools/postcss-hwb-function",
    "@csstools/postcss-font-format-keywords",
    "@csstools/postcss-normalize-display-values",
    "fast-diff",
    "@csstools/postcss-oklab-function",
    "@csstools/postcss-color-function",
    "@csstools/postcss-ic-unit",
    "resolve-pathname",
    "@csstools/postcss-stepped-value-functions",
    "@csstools/postcss-unset-value",
    "@scure/bip32",
    "npm-conf",
    "split",
    "@types/jsonwebtoken",
    "@esbuild/android-arm",
    "unified",
    "cssnano-util-get-arguments",
    "gtoken",
    "babel-plugin-transform-exponentiation-operator",
    "fetch-blob",
    "cssnano-util-get-match",
    "get-port",
    "d3-timer",
    "@aws-sdk/nested-clients",
    "@csstools/postcss-cascade-layers",
    "babel-plugin-styled-components",
    "@csstools/postcss-trigonometric-functions",
    "@csstools/postcss-nested-calc",
    "@csstools/postcss-text-decoration-shorthand",
    "memoizee",
    "babel-polyfill",
    "b4a",
    "@typespec/ts-http-runtime",
    "cssnano-util-same-parent",
    "redis-parser",
    "value-equal",
    "@radix-ui/react-compose-refs",
    "@emotion/styled",
    "@types/mdast",
    "cssnano-util-raw-cache",
    "editions",
    "@aws-sdk/util-arn-parser",
    "quick-format-unescaped",
    "cookiejar",
    "lodash.repeat",
    "ethereum-cryptography",
    "opener",
    "lodash.assign",
    "after",
    "@aws-cdk/cloud-assembly-schema",
    "atomic-sleep",
    "rfdc",
    "@balena/dockerignore",
    "fast-fifo",
    "@cnakazawa/watch",
    "isbinaryfile",
    "parse-filepath",
    "mri",
    "archiver",
    "@azure/core-util",
    "d3-format",
    "pino-std-serializers",
    "path",
    "@esbuild/win32-x64",
    "d3-ease",
    "@esbuild/linux-x64",
    "@esbuild/darwin-arm64",
    "parse5-parser-stream",
    "encoding-sniffer",
    "google-auth-library",
    "@esbuild/darwin-x64",
    "d3-path",
    "@esbuild/linux-arm64",
    "@esbuild/linux-arm",
    "@esbuild/android-arm64",
    "@esbuild/android-x64",
    "@esbuild/freebsd-arm64",
    "@esbuild/freebsd-x64",
    "@esbuild/linux-ia32",
    "@esbuild/linux-mips64el",
    "@esbuild/linux-ppc64",
    "@esbuild/linux-riscv64",
    "@esbuild/linux-s390x",
    "@esbuild/netbsd-x64",
    "@esbuild/openbsd-x64",
    "@esbuild/sunos-x64",
    "@esbuild/win32-arm64",
    "@esbuild/win32-ia32",
    "@hapi/address",
    "d3-time",
    "form-data-encoder",
    "@radix-ui/react-slot",
    "xregexp",
    "bs58check",
    "d3-time-format",
    "read-cmd-shim",
    "url-join",
    "d3-shape",
    "blob",
    "@apideck/better-ajv-errors",
    "workbox-recipes",
    "timers-ext",
    "@ant-design/colors",
    "yeast",
    "has-cors",
    "formdata-polyfill",
    "parseuri",
    "string-convert",
    "sonic-boom",
    "constructs",
    "parseqs",
    "@babel/plugin-proposal-optional-catch-binding",
    "json2mq",
    "lru-queue",
    "pg-connection-string",
    "postcss-safe-parser",
    "gaxios",
    "pngjs",
    "deep-eql",
    "fast-redact",
    "arraybuffer.slice",
    "secp256k1",
    "@ethersproject/sha2",
    "compute-scroll-into-view",
    "d3-scale",
    "fastest-levenshtein",
    "@aws-cdk/cx-api",
    "needle",
    "pino",
    "@hapi/joi",
    "uc.micro",
    "buffer-equal",
    "ext-list",
    "esprima-fb",
    "xmldom",
    "mdast-util-to-string",
    "fastparse",
    "ext-name",
    "@babel/plugin-transform-react-jsx-source",
    "options",
    "@npmcli/promise-spawn",
    "@azure/logger",
    "@grpc/proto-loader",
    "string-template",
    "pretty-hrtime",
    "pause-stream",
    "assertion-error",
    "remark-parse",
    "d3-dispatch",
    "flagged-respawn",
    "linkify-it",
    "superagent",
    "array-each",
    "ansi",
    "@smithy/util-waiter",
    "@aws-cdk/region-info",
    "toposort",
    "fs",
    "stream-combiner",
    "morgan",
    "merge",
    "strip-bom-buf",
    "@radix-ui/react-primitive",
    "through2-filter",
    "serialize-error",
    "keccak",
    "@octokit/openapi-types",
    "reselect",
    "rx-lite-aggregates",
    "p-each-series",
    "object.defaults",
    "rx",
    "@ant-design/icons",
    "component-inherit",
    "is-url",
    "lit-html",
    "fdir",
    "blakejs",
    "@jest/get-type",
    "rw",
    "@jest/diff-sequences",
    "make-iterator",
    "callsite",
    "babel-helper-vue-jsx-merge-props",
    "@types/bn.js",
    "dom-align",
    "is-what",
    "optimize-css-assets-webpack-plugin",
    "mpath",
    "@jest/pattern",
    "last-call-webpack-plugin",
    "cluster-key-slot",
    "sentence-case",
    "read-package-json-fast",
    "@npmcli/git",
    "sort-keys-length",
    "pkginfo",
    "@aws-cdk/core",
    "@ethersproject/random",
    "rollup-pluginutils",
    "postcss-values-parser",
    "@ethersproject/basex",
    "component-bind",
    "mocha",
    "mongoose",
    "buffer-shims",
    "void-elements",
    "promzard",
    "obliterator",
    "@babel/plugin-transform-react-jsx-self",
    "@azure/core-auth",
    "es6-set",
    "d3-selection",
    "mquery",
    "@npmcli/node-gyp",
    "to-array",
    "css-selector-tokenizer",
    "gcp-metadata",
    "@octokit/types",
    "rc-tooltip",
    "liftoff",
    "chai",
    "fined",
    "circular-json",
    "parse-headers",
    "@grpc/grpc-js",
    "to-absolute-glob",
    "change-case",
    "kareem",
    "constant-case",
    "pnp-webpack-plugin",
    "postcss-selector-matches",
    "lit-element",
    "@sideway/formula",
    "@sideway/address",
    "@sideway/pinpoint",
    "space-separated-tokens",
    "fstream-ignore",
    "less",
    "@babel/plugin-proposal-async-generator-functions",
    "streamx",
    "@npmcli/run-script",
    "redis-errors",
    "mustache",
    "path-case",
    "bare-events",
    "@types/keygrip",
    "@xmldom/xmldom",
    "property-information",
    "envinfo",
    "figlet",
    "process-warning",
    "comma-separated-tokens",
    "@azure/core-tracing",
    "sass-graph",
    "object.map",
    "yeoman-generator",
    "postgres-array",
    "postgres-interval",
    "postgres-date",
    "postgres-bytea",
    "scss-tokenizer",
    "ufo",
    "async-foreach",
    "@radix-ui/react-use-callback-ref",
    "luxon",
    "@radix-ui/react-use-layout-effect",
    "crc",
    "bplist-parser",
    "@babel/helper-environment-visitor",
    "ccount",
    "node-sass",
    "@npmcli/installed-package-contents",
    "@ethersproject/providers",
    "bin-links",
    "mitt",
    "ts-pnp",
    "vinyl-file",
    "pg-int8",
    "workerpool",
    "rc-slider",
    "typedarray.prototype.slice",
    "delay",
    "@ethersproject/pbkdf2",
    "@ethersproject/wordlists",
    "@ethersproject/solidity",
    "true-case-path",
    "@parcel/watcher",
    "readline2",
    "to-readable-stream",
    "@floating-ui/react-dom",
    "d3-transition",
    "pn",
    "@ethersproject/hdnode",
    "micromark",
    "cli-table3",
    "rc-progress",
    "text-decoder",
    "cookie-parser",
    "eslint-config-prettier",
    "@aws-sdk/util-base64-browser",
    "rc-resize-observer",
    "has-color",
    "@ethersproject/json-wallets",
    "postcss-color-gray",
    "p-queue",
    "es6-map",
    "@types/hast",
    "@smithy/eventstream-serde-universal",
    "@ethersproject/wallet",
    "@opentelemetry/api",
    "universal-user-agent",
    "mdast-util-from-markdown",
    "@csstools/convert-colors",
    "@types/cors",
    "jszip",
    "@ethersproject/contracts",
    "tildify",
    "is-valid-glob",
    "zwitch",
    "read-all-stream",
    "@socket.io/component-emitter",
    "postcss-color-mod-function",
    "markdown-it",
    "@mapbox/node-pre-gyp",
    "detect-node-es",
    "sift",
    "rc-menu",
    "d3-geo",
    "debuglog",
    "plist",
    "scroll-into-view-if-needed",
    "@types/webidl-conversions",
    "@radix-ui/react-context",
    "tinyglobby",
    "@types/whatwg-url",
    "rc-checkbox",
    "pg-types",
    "rc-collapse",
    "rc-tree",
    "read-chunk",
    "rc-dropdown",
    "xhr",
    "bootstrap",
    "use-callback-ref",
    "init-package-json",
    "google-protobuf",
    "use-sidecar",
    "normalize-wheel",
    "sass",
    "github-username",
    "rc-select",
    "sqlstring",
    "longest-streak",
    "@types/tough-cookie",
    "rc-dialog",
    "@remix-run/router",
    "rc-input-number",
    "varint",
    "header-case",
    "@ethersproject/units",
    "rc-notification",
    "readdir-glob",
    "rc-tabs",
    "cli-truncate",
    "@parcel/watcher-linux-x64-glibc",
    "p-reduce",
    "@parcel/watcher-win32-x64",
    "@parcel/watcher-android-arm64",
    "@parcel/watcher-darwin-arm64",
    "@parcel/watcher-darwin-x64",
    "@parcel/watcher-freebsd-x64",
    "@parcel/watcher-linux-arm-glibc",
    "@parcel/watcher-linux-arm64-glibc",
    "@parcel/watcher-linux-arm64-musl",
    "@parcel/watcher-linux-x64-musl",
    "@parcel/watcher-win32-arm64",
    "@parcel/watcher-win32-ia32",
    "@parcel/watcher-linux-arm-musl",
    "mongodb-connection-string-url",
    "rc-pagination",
    "prismjs",
    "is-hex-prefixed",
    "rc-table",
    "rc-upload",
    "escope",
    "@radix-ui/primitive",
    "rc-switch",
    "redux-thunk",
    "strip-hex-prefix",
    "rarerteat",
    "vuex",
    "rc-steps",
    "git-clone",
    "d3-drag",
    "rc-tree-select",
    "token-types",
    "strtok3",
    "@tokenizer/token",
    "rc-cascader",
    "mem-fs-editor",
    "@angular/core",
    "rc-rate",
    "node-emoji",
    "@aws-cdk/aws-iam",
    "esm",
    "real-require",
    "redeyed",
    "d3-hierarchy",
    "tehtehteh",
    "microevent.ts",
    "glob-watcher",
    "log-update",
    "worker-rpc",
    "lodash._bindcallback",
    "columnify",
    "google-p12-pem",
    "redis",
    "browser-stdout",
    "pino-abstract-transport",
    "in-publish",
    "qrcode-terminal",
    "check-error",
    "webpack-merge",
    "stubs",
    "@octokit/request-error",
    "utf8",
    "@types/tapable",
    "pathval",
    "stdout-stream",
    "on-exit-leak-free",
    "@octokit/endpoint",
    "rc-drawer",
    "touch",
    "babel-extract-comments",
    "eventemitter2",
    "rc-align",
    "antd",
    "deprecation",
    "multer",
    "graphql-tag",
    "mixin-object",
    "@mongodb-js/saslprep",
    "babel-preset-env",
    "d3-dsv",
    "d3-zoom",
    "rc-trigger",
    "thread-stream",
    "get-nonce",
    "left-pad",
    "@ethereumjs/rlp",
    "array-tree-filter",
    "append-field",
    "rc-motion",
    "map-stream",
    "@radix-ui/react-use-controllable-state",
    "zone.js",
    "temp",
    "multiformats",
    "pgpass",
    "gud",
    "mnemonist",
    "netmask",
    "stream-combiner2",
    "eslint-loader",
    "rlp",
    "@angular/common",
    "sliced",
    "rc-mentions",
    "cardinal",
    "grouped-queue",
    "mem-fs",
    "@babel/register",
    "d3-quadtree",
    "npm-user-validate",
    "is-in-browser",
    "react-style-singleton",
    "@octokit/request",
    "@types/scheduler",
    "@esbuild/aix-ppc64",
    "@mui/types",
    "adm-zip",
    "zod-to-json-schema",
    "d3",
    "caniuse-db",
    "set-immediate-shim",
    "@mui/utils",
    "d3-force",
    "rework",
    "superstruct",
    "express-validator",
    "@adraffy/ens-normalize",
    "stream-events",
    "unist-util-position",
    "cli-color",
    "@graphql-typed-document-node/core",
    "nwmatcher",
    "reduce-css-calc",
    "loupe",
    "@ant-design/icons-svg",
    "react-remove-scroll-bar",
    "growl",
    "pause",
    "before-after-hook",
    "inline-style-parser",
    "@babel/polyfill",
    "pathe",
    "acorn-dynamic-import",
    "rework-visit",
    "style-to-object",
    "errlop",
    "jju",
    "plugin-error",
    "vlq",
    "babel-plugin-syntax-class-properties",
    "babel-plugin-syntax-flow",
    "readdir-scoped-modules",
    "lodash.pick",
    "get-func-name",
    "dompurify",
    "pg-pool",
    "output-file-sync",
    "http-response-object",
    "node-pre-gyp",
    "is-svg",
    "sharp",
    "loader-fs-cache",
    "@types/webpack",
    "aria-hidden",
    "@sigstore/protobuf-specs",
    "@radix-ui/react-presence",
    "@lit-labs/ssr-dom-shim",
    "react-remove-scroll",
    "error",
    "@types/uglify-js",
    "@tufjs/canonical-json",
    "yeoman-environment",
    "@aws-sdk/util-base64-node",
    "base64url",
    "@types/secp256k1",
    "lodash.foreach",
    "os-name",
    "babel-helper-builder-react-jsx",
    "@lit/reactive-element",
    "koa-compose",
    "style-equal",
    "@types/source-list-map",
    "standard-as-callback",
    "element-ui",
    "@mui/private-theming",
    "coffee-script",
    "download-git-repo",
    "xmlhttprequest",
    "pg",
    "@angular/platform-browser",
    "@mui/styled-engine",
    "shallow-element-equals",
    "@npmcli/package-json",
    "babel-plugin-transform-class-properties",
    "babel-plugin-transform-flow-strip-types",
    "@oozcitak/util",
    "@types/webpack-sources",
    "@alifd/next",
    "node-status-codes",
    "dicer",
    "is-nan",
    "@types/pbkdf2",
    "find-index",
    "micromark-util-symbol",
    "@tufjs/models",
    "@mui/system",
    "lit",
    "json-loader",
    "npm",
    "lodash.pad",
    "minipass-json-stream",
    "jss",
    "ethereumjs-util",
    "@angular/compiler",
    "arity-n",
    "micromark-util-types",
    "compose-function",
    "@emnapi/runtime",
    "walk-up-path",
    "zen-observable",
    "@babel/plugin-proposal-unicode-property-regex",
    "babel-plugin-transform-react-jsx",
    "@babel/plugin-proposal-json-strings",
    "require-uncached",
    "@ioredis/commands",
    "@aws-crypto/crc32c",
    "@octokit/auth-token",
    "@npmcli/name-from-folder",
    "vfile-location",
    "common-ancestor-path",
    "nodemailer",
    "borsh",
    "prettier-linter-helpers",
    "contains-path",
    "isemail",
    "@ctrl/tinycolor",
    "urijs",
    "hast-util-parse-selector",
    "@rollup/rollup-linux-x64-gnu",
    "cookie-session",
    "tuf-js",
    "realpath-native",
    "@types/yauzl",
    "markdown-table",
    "@sigstore/bundle",
    "@angular/forms",
    "d3-axis",
    "clap",
    "fflate",
    "@mui/material",
    "mdast-util-to-hast",
    "micromark-util-character",
    "hastscript",
    "@mui/core-downloads-tracker",
    "copy-anything",
    "degenerator",
    "react-popper",
    "d3-scale-chromatic",
    "@angular/platform-browser-dynamic",
    "css-vendor",
    "lodash.isobject",
    "babel-plugin-syntax-dynamic-import",
    "math-expression-evaluator",
    "node-fetch-npm",
    "yaeti",
    "ioredis",
    "@aws-sdk/middleware-sdk-s3",
    "reduce-function-call",
    "router",
    "topo",
    "just-diff",
    "shallow-equal",
    "parse-cache-control",
    "better-assert",
    "@radix-ui/react-id",
    "@sigstore/tuf",
    "commoner",
    "just-diff-apply",
    "d3-brush",
    "get-uri",
    "uid-safe",
    "d3-random",
    "@fastify/busboy",
    "rc-virtual-list",
    "glob2base",
    "whet.extend",
    "@sigstore/sign",
    "@ethereumjs/util",
    "@npmcli/map-workspaces",
    "pac-resolver",
    "@types/caseless",
    "os-shim",
    "tldts-core",
    "pg-protocol",
    "d3-polygon",
    "inflection",
    "internmap",
    "websocket",
    "tunnel",
    "@inquirer/core",
    "buffers",
    "@inquirer/type",
    "babel-plugin-transform-react-display-name",
    "has-binary2",
    "rc-overflow",
    "copy-to",
    "sigstore",
    "command-exists",
    "rc-field-form",
    "@angular/router",
    "tailwind-merge",
    "lodash.map",
    "mdast-util-to-markdown",
    "gh-got",
    "event-stream",
    "stream-buffers",
    "d3-chord",
    "spawn-sync",
    "pac-proxy-agent",
    "tldts",
    "@smithy/eventstream-serde-browser",
    "async-retry",
    "@smithy/eventstream-serde-config-resolver",
    "@smithy/eventstream-serde-node",
    "ylru",
    "babel-plugin-transform-react-jsx-source",
    "iterall",
    "defu",
    "object-path",
    "eslint-plugin-prettier",
    "parse-conflict-json",
    "@octokit/plugin-paginate-rest",
    "cidr-regex",
    "web3-utils",
    "chainsaw",
    "treeverse",
    "gulp-rename",
    "json-stringify-nice",
    "promise-all-reject-late",
    "promise-call-limit",
    "saslprep",
    "teeny-request",
    "@aws-cdk/aws-kms",
    "uint8arrays",
    "add-dom-event-listener",
    "shimmer",
    "jsonc-parser",
    "@npmcli/metavuln-calculator",
    "@octokit/graphql",
    "micromark-util-encode",
    "i",
    "throttleit",
    "array-filter",
    "micromark-util-sanitize-uri",
    "stdin-discarder",
    "ignore-by-default",
    "@types/fs-extra",
    "hasha",
    "text-encoding-utf-8",
    "set-getter",
    "react-icons",
    "stream-chain",
    "character-entities-html4",
    "html-comment-regex",
    "stringify-entities",
    "rc-picker",
    "decode-named-character-reference",
    "@isaacs/string-locale-compare",
    "request-promise",
    "is-gzip",
    "secure-json-parse",
    "only",
    "rc-textarea",
    "@types/cookie-session",
    "lodash.flattendeep",
    "title-case",
    "http-assert",
    "stream-json",
    "random-bytes",
    "intl-messageformat",
    "postcss-message-helpers",
    "@types/request",
    "d3-collection",
    "@radix-ui/react-use-size",
    "micromark-util-decode-numeric-character-reference",
    "lodash.padstart",
    "koa",
    "proxy-agent",
    "confbox",
    "@ant-design/react-slick",
    "rc-image",
    "micro-ftch",
    "pkg-types",
    "@rc-component/portal",
)

MSG_NOT_AUTHENTICATED_TOOL = "{tool_name} is aliased to Safety to ensure your package installations are audited and secured."
MSG_NOT_AUTHENTICATED_TOOL_NO_TTY = "Safety is not authenticated - this usage of {tool_name} will not be audited by Safety. Please run safety auth login or set an environment variable for SAFETY_API_KEY if running in a script."



import abc
import os.path
import re
from pathlib import Path
from sys import platform


from safety.tool.pip import Pip
from safety.tool.poetry import Poetry
from safety.tool.npm import Npm

from typing import Any, TYPE_CHECKING, Optional

from safety.tool.uv.main import Uv

if TYPE_CHECKING:
    pass


def is_os_supported():
    return platform in ["linux", "linux2", "darwin", "win32"]


class BuildFileConfigurator(abc.ABC):
    @abc.abstractmethod
    def is_supported(self, file: Path) -> bool:
        """
        Returns whether a specific file is supported by this class.
        Args:
            file (str): The file to check.
        Returns:
            bool: Whether the file is supported by this class.
        """
        pass

    @abc.abstractmethod
    def configure(
        self, file: Path, org_slug: Optional[str], project_id: Optional[str]
    ) -> Optional[Path]:
        """
        Configures specific file.
        Args:
            file (str): The file to configure.
            org_slug (str): The organization slug.
            project_id (str): The project identifier.
        """
        pass


class PipRequirementsConfigurator(BuildFileConfigurator):
    __file_name_pattern = re.compile("^([a-zA-Z_-]+)?requirements([a-zA-Z_-]+)?.txt$")

    def is_supported(self, file: Path) -> bool:
        return self.__file_name_pattern.match(os.path.basename(file)) is not None

    def configure(
        self, file: Path, org_slug: Optional[str], project_id: Optional[str]
    ) -> None:
        Pip.configure_requirements(file, org_slug, project_id)


class PoetryPyprojectConfigurator(BuildFileConfigurator):
    __file_name_pattern = re.compile("^pyproject.toml$")

    def is_supported(self, file: Path) -> bool:
        return self.__file_name_pattern.match(
            os.path.basename(file)
        ) is not None and Poetry.is_poetry_project_file(file)

    def configure(
        self, file: Path, org_slug: Optional[str], project_id: Optional[str]
    ) -> Optional[Path]:
        if self.is_supported(file):
            return Poetry.configure_pyproject(file, org_slug, project_id)  # type: ignore
        return None


# TODO: Review if we should move this/hook up this into interceptors.
class ToolConfigurator(abc.ABC):
    @abc.abstractmethod
    def configure(self, org_slug: Optional[str]) -> Any:
        """
        Configures specific tool.
        Args:
            org_slug (str): The organization slug.
        """
        pass

    @abc.abstractmethod
    def reset(self) -> None:
        """
        Resets specific tool.
        """
        pass


class PipConfigurator(ToolConfigurator):
    def configure(self, org_slug: Optional[str]) -> Optional[Path]:
        return Pip.configure_system(org_slug)

    def reset(self) -> None:
        Pip.reset_system()


class PoetryConfigurator(ToolConfigurator):
    """
    Configures poetry system is not supported.
    """

    def configure(self, org_slug: Optional[str]) -> Optional[Path]:
        return None

    def reset(self) -> None:
        return None


class UvConfigurator(ToolConfigurator):
    def configure(self, org_slug: Optional[str]) -> Optional[Path]:
        return Uv.configure_system(org_slug)

    def reset(self) -> None:
        Uv.reset_system()


class UvPyprojectConfigurator(BuildFileConfigurator):
    __file_name_pattern = re.compile("^uv.lock$")

    def is_supported(self, file: Path) -> bool:
        return (
            self.__file_name_pattern.match(os.path.basename(file)) is not None
            and Path("pyproject.toml").exists()
        )

    def configure(
        self, file: Path, org_slug: Optional[str], project_id: Optional[str]
    ) -> Optional[Path]:
        if self.is_supported(file):
            return Uv.configure_pyproject(Path("pyproject.toml"), org_slug, project_id)
        return None


class NpmConfigurator(ToolConfigurator):
    def configure(self, org_slug: Optional[str]) -> Optional[Path]:
        return Npm.configure_system(org_slug)

    def reset(self) -> None:
        Npm.reset_system()


class NpmProjectConfigurator(BuildFileConfigurator):
    __file_name_pattern = re.compile("^package.json$")

    def is_supported(self, file: Path) -> bool:
        return self.__file_name_pattern.match(os.path.basename(file)) is not None

    def configure(
        self, file: Path, org_slug: Optional[str], project_id: Optional[str]
    ) -> Optional[Path]:
        if self.is_supported(file):
            return Npm.configure_project(file, org_slug, project_id)
        return None



"""
Test suite for TyposquattingProtection functionality.
"""

import pytest
from unittest.mock import MagicMock, patch

from safety.tool.typosquatting import TyposquattingProtection
from safety.tool.intents import CommandToolIntention, ToolIntentionType


@pytest.mark.unit
class TestTyposquattingProtection:
    """Test typosquatting protection functionality"""

    def setup_method(self):
        """Set up test fixtures"""
        # Use a small set of popular packages for testing
        self.popular_packages = [
            "requests",
            "flask",
            "django",
            "numpy",
            "pandas",
            "scipy",
            "matplotlib",
            "tensorflow",
            "pytorch",
            "boto3",
        ]
        self.protection = TyposquattingProtection(self.popular_packages)

    def test_check_package_exact_match_returns_valid(self):
        """Test that exact matches for popular packages return valid"""
        is_valid, suggestion = self.protection.check_package("requests")

        assert is_valid is True
        assert suggestion == "requests"

    def test_check_package_typo_returns_invalid_with_suggestion(self):
        """Test that typos return invalid with correct suggestion"""
        is_valid, suggestion = self.protection.check_package("reqests")  # missing 'u'

        assert is_valid is False
        assert suggestion == "requests"

    def test_check_package_similar_name_suggests_popular(self):
        """Test that similar names suggest popular packages"""
        is_valid, suggestion = self.protection.check_package("flaskk")  # extra 'k'

        assert is_valid is False
        assert suggestion == "flask"

    def test_check_package_edit_distance_threshold(self):
        """Test edit distance threshold based on package name length"""
        # Short names (<=5 chars) have threshold of 1
        is_valid, suggestion = self.protection.check_package(
            "flak"
        )  # edit distance 1 from "flask"
        assert is_valid is False
        assert suggestion == "flask"

        # Longer names have threshold of 2
        is_valid, suggestion = self.protection.check_package(
            "requets"
        )  # edit distance 2 from "requests"
        assert is_valid is False
        assert suggestion == "requests"

    def test_check_package_unknown_package_returns_valid(self):
        """Test that unknown packages (not typos) return valid"""
        is_valid, suggestion = self.protection.check_package(
            "completely-unknown-package"
        )

        assert is_valid is True
        assert suggestion == "completely-unknown-package"

    def test_check_package_length_difference_too_large(self):
        """Test that packages with large length differences are not flagged"""
        is_valid, suggestion = self.protection.check_package(
            "req"
        )  # too short compared to "requests"

        assert is_valid is True
        assert suggestion == "req"

    def test_coerce_add_intention_prompts_with_install(self):
        """Test ADD intention prompts with 'install' verb"""
        intention = MagicMock(spec=CommandToolIntention)
        intention.intention_type = ToolIntentionType.ADD_PACKAGE

        with patch("safety.tool.typosquatting.Prompt.ask") as mock_prompt:
            mock_prompt.return_value = "y"

            result = self.protection.coerce(intention, "reqests")  # typo

            # Should ask about installing and return corrected name
            mock_prompt.assert_called_once()
            call_args = mock_prompt.call_args
            assert "install reqests" in call_args[1]["prompt"]
            assert "install requests" in call_args[1]["prompt"]
            assert result == "requests"

    def test_coerce_download_intention_prompts_with_download(self):
        """Test DOWNLOAD intention prompts with 'download' verb"""
        intention = MagicMock(spec=CommandToolIntention)
        intention.intention_type = ToolIntentionType.DOWNLOAD_PACKAGE

        with patch("safety.tool.typosquatting.Prompt.ask") as mock_prompt:
            mock_prompt.return_value = "y"

            result = self.protection.coerce(intention, "flsk")  # typo for flask

            # Should ask about downloading
            mock_prompt.assert_called_once()
            call_args = mock_prompt.call_args
            assert "download flsk" in call_args[1]["prompt"]
            assert "download flask" in call_args[1]["prompt"]
            assert result == "flask"

    def test_coerce_build_intention_prompts_with_build(self):
        """Test BUILD intention prompts with 'build' verb"""
        intention = MagicMock(spec=CommandToolIntention)
        intention.intention_type = ToolIntentionType.BUILD_PROJECT

        with patch("safety.tool.typosquatting.Prompt.ask") as mock_prompt:
            mock_prompt.return_value = "y"

            result = self.protection.coerce(intention, "djnago")  # typo for django

            # Should ask about building
            mock_prompt.assert_called_once()
            call_args = mock_prompt.call_args
            assert "build djnago" in call_args[1]["prompt"]
            assert "build django" in call_args[1]["prompt"]
            assert result == "django"

    def test_coerce_user_says_no_returns_original(self):
        """Test that user saying 'no' returns original package name"""
        intention = MagicMock(spec=CommandToolIntention)
        intention.intention_type = ToolIntentionType.ADD_PACKAGE

        with patch("safety.tool.typosquatting.Prompt.ask") as mock_prompt:
            mock_prompt.return_value = "n"

            result = self.protection.coerce(intention, "reqests")

            # Should return original typo since user said no
            assert result == "reqests"

    def test_coerce_valid_package_no_prompt(self):
        """Test that valid packages don't trigger prompts"""
        intention = MagicMock(spec=CommandToolIntention)
        intention.intention_type = ToolIntentionType.ADD_PACKAGE

        with patch("safety.tool.typosquatting.Prompt.ask") as mock_prompt:
            result = self.protection.coerce(intention, "requests")

            # Should not prompt for valid packages
            mock_prompt.assert_not_called()
            assert result == "requests"

    def test_coerce_unknown_package_no_prompt(self):
        """Test that unknown packages don't trigger prompts"""
        intention = MagicMock(spec=CommandToolIntention)
        intention.intention_type = ToolIntentionType.ADD_PACKAGE

        with patch("safety.tool.typosquatting.Prompt.ask") as mock_prompt:
            result = self.protection.coerce(intention, "my-custom-package")

            # Should not prompt for unknown packages
            mock_prompt.assert_not_called()
            assert result == "my-custom-package"

    def test_coerce_prompt_defaults_to_yes(self):
        """Test that the prompt defaults to 'yes'"""
        intention = MagicMock(spec=CommandToolIntention)
        intention.intention_type = ToolIntentionType.ADD_PACKAGE

        with patch("safety.tool.typosquatting.Prompt.ask") as mock_prompt:
            mock_prompt.return_value = "y"

            self.protection.coerce(intention, "reqests")

            # Check that prompt was called with correct defaults
            call_args = mock_prompt.call_args
            assert call_args[1]["default"] == "y"
            assert call_args[1]["show_default"] is True
            assert call_args[1]["choices"] == ["y", "n"]

    def test_coerce_case_insensitive_response(self):
        """Test that user response is case insensitive"""
        intention = MagicMock(spec=CommandToolIntention)
        intention.intention_type = ToolIntentionType.ADD_PACKAGE

        with patch("safety.tool.typosquatting.Prompt.ask") as mock_prompt:
            # Test uppercase Y
            mock_prompt.return_value = "Y"
            result = self.protection.coerce(intention, "reqests")
            assert result == "requests"  # Should correct the typo

            # Test uppercase N
            mock_prompt.return_value = "N"
            result = self.protection.coerce(intention, "reqests")
            assert result == "reqests"  # Should keep original

    def test_multiple_similar_packages_suggests_closest(self):
        """Test that closest match is suggested when multiple similar packages exist"""
        # Add similar packages to test closest match selection
        packages_with_similar = self.popular_packages + ["request", "requests-oauthlib"]
        protection = TyposquattingProtection(packages_with_similar)

        # "reqests" is closest to "requests" (1 edit) vs "request" (2 edits)
        is_valid, suggestion = protection.check_package("reqests")

        assert is_valid is False
        assert suggestion == "requests"

    def test_nltk_edit_distance_calculation(self):
        """Test that NLTK edit distance is used correctly"""
        # Test known edit distances
        with patch(
            "safety.tool.typosquatting.nltk.edit_distance"
        ) as mock_edit_distance:
            mock_edit_distance.return_value = 1

            is_valid, suggestion = self.protection.check_package("reqests")

            # Should have called nltk.edit_distance
            mock_edit_distance.assert_called()
            assert is_valid is False

    @pytest.mark.parametrize(
        "package_name,expected_valid,expected_suggestion",
        [
            ("requests", True, "requests"),  # Exact match
            ("reqests", False, "requests"),  # Missing 'u'
            ("flask", True, "flask"),  # Exact match
            ("flsk", False, "flask"),  # Missing 'a'
            ("django", True, "django"),  # Exact match
            ("djnago", False, "django"),  # Swapped 'a' and 'n'
            (
                "completely-new-package",
                True,
                "completely-new-package",
            ),  # Unknown package
        ],
    )
    def test_check_package_various_cases(
        self, package_name, expected_valid, expected_suggestion
    ):
        """Test various package name checking scenarios"""
        is_valid, suggestion = self.protection.check_package(package_name)

        assert is_valid == expected_valid
        assert suggestion == expected_suggestion

    def test_console_parameter_passed_to_prompt(self):
        """Test that console parameter is passed to Rich Prompt"""
        intention = MagicMock(spec=CommandToolIntention)
        intention.intention_type = ToolIntentionType.ADD_PACKAGE

        with patch("safety.tool.typosquatting.Prompt.ask") as mock_prompt:
            mock_prompt.return_value = "y"

            self.protection.coerce(intention, "reqests")

            # Check that console parameter was passed
            call_args = mock_prompt.call_args
            assert "console" in call_args[1]



from importlib.metadata import PackageNotFoundError, version
import logging
import os
import platform
from typing import Dict, Optional


LOG = logging.getLogger(__name__)


def get_version() -> Optional[str]:
    """
    Get the version of the Safety package.

    Returns:
      Optional[str]: The Safety version if found, otherwise None.
    """
    try:
        return version("safety")
    except PackageNotFoundError:
        LOG.exception("Unable to get Safety version.")
        return None


def get_identifier() -> str:
    """
    Get the identifier of the source type.

    Returns:
      str: The source type identifier.
    """

    if source := os.environ.get("SAFETY_SOURCE_TYPE", None):
        return source

    from safety_schemas.models.events.types import SourceType

    return SourceType.SAFETY_CLI_PYPI.value


def get_user_agent() -> str:
    """
    Get the user agent string for HTTP requests.

    Returns:
      str: The user agent string in the format: safety-cli/{version} ({os} {arch}; Python/{python_version})
    """
    safety_version = get_version() or "unknown"
    os_name = platform.system()

    # Get architecture
    machine = platform.machine()
    # Normalize architecture names
    if machine in ("x86_64", "AMD64"):
        arch = "x86_64"
    elif machine in ("arm64", "aarch64"):
        arch = "arm_64"
    elif machine == "i386":
        arch = "x86"
    else:
        arch = machine or "unknown"

    python_version = platform.python_version()

    return f"safety-cli/{safety_version} ({os_name} {arch}; Python/{python_version})"


def get_meta_http_headers() -> Dict[str, str]:
    """
    Get the metadata headers for the client.

    Returns:
      Dict[str, str]: The metadata headers.
    """

    from safety_schemas.models.events.constants import SAFETY_NAMESPACE

    namespace = SAFETY_NAMESPACE.title()

    return {
        f"{namespace}-Client-Version": get_version() or "",
        f"{namespace}-Client-Id": get_identifier(),
        "User-Agent": get_user_agent(),
    }



from typing import Dict, Optional, Union, Set

from ..base import ToolCommandLineParser
from ..intents import Dependency, ToolIntentionType


class PoetryParser(ToolCommandLineParser):
    def get_tool_name(self) -> str:
        return "poetry"

    def get_command_hierarchy(self) -> Dict[str, Union[ToolIntentionType, Dict]]:
        """
        Allow base parser to recognize poetry commands and intentions.
        """
        return {
            "add": ToolIntentionType.ADD_PACKAGE,
            "remove": ToolIntentionType.REMOVE_PACKAGE,
            "update": ToolIntentionType.UPDATE_PACKAGE,
            "install": ToolIntentionType.SYNC_PACKAGES,
            "build": ToolIntentionType.BUILD_PROJECT,
            "show": ToolIntentionType.LIST_PACKAGES,
            "init": ToolIntentionType.INIT_PROJECT,
        }

    def get_known_flags(self) -> Dict[str, Set[str]]:
        """
        Flags that DO NOT take a value, derived from `poetry --help` and subcommand helps.
        """
        return {
            "global": {
                "help",
                "h",
                "quiet",
                "q",
                "version",
                "V",
                "ansi",
                "no-ansi",
                "no-interaction",
                "n",
                "no-plugins",
                "no-cache",
                "verbose",
                "v",
                "vv",
                "vvv",
            },
            "add": {
                "dev",
                "D",
                "editable",
                "e",
                "allow-prereleases",
                "dry-run",
                "lock",
            },
            "remove": {
                "dev",
                "D",
                "dry-run",
                "lock",
            },
            "update": {
                "sync",
                "dry-run",
                "lock",
            },
            "install": {
                "sync",
                "no-root",
                "no-directory",
                "dry-run",
                "all-extras",
                "all-groups",
                "only-root",
                "compile",
            },
            "build": {
                "clean",
            },
        }

    def _parse_package_spec(
        self, spec_str: str, arg_index: int
    ) -> Optional[Dependency]:
        """
        Parse a package specification string into a Dependency object.
        Handles various formats including Poetry-specific syntax and standard PEP 508 requirements.

        Args:
            spec_str: Package specification string (e.g. "requests>=2.25.0[security]")

        Returns:
            Dependency: Parsed dependency information

        Raises:
            ValueError: If the specification cannot be parsed
        """
        try:
            # TODO: This is a very basic implementation and not well tested
            # our main target for now is to get the package name.
            from packaging.requirements import Requirement

            include_specifier = False

            # Handle @ operator (package@version)
            if "@" in spec_str and not spec_str.startswith("git+"):
                name = spec_str.split("@")[0]

            # Handle caret requirements (package^version)
            elif "^" in spec_str:
                name = spec_str.split("^")[0]

            # Handle tilde requirements (package~version)
            elif "~" in spec_str and "~=" not in spec_str:
                name = spec_str.split("~")[0]

            else:
                # Common PEP 440 cases
                name = spec_str
                include_specifier = True

            req = Requirement(name)

            return Dependency(
                name=req.name,
                version_constraint=str(req.specifier) if include_specifier else None,
                extras=req.extras,
                arg_index=arg_index,
                original_text=spec_str,
            )
        except Exception:
            # If spec parsing fails, just ignore for now
            return None



import sys
import subprocess
import shutil
import logging

from safety.utils.pyapp_utils import get_path, get_env

logger = logging.getLogger(__name__)


def get_unwrapped_command(name: str) -> str:
    """
    Find the true executable for a command, skipping wrappers/aliases/.bat files.

    Args:
        command: The command to resolve (e.g. 'pip', 'python')

    Returns:
        Path to the actual executable
    """
    logger.debug(f"get_unwrapped_command called with name: {name}")

    if sys.platform in ["win32"]:
        for lookup_term in [f"{name}.exe", name]:
            logger.debug(f"Windows platform detected, looking for: {lookup_term}")

            where_result = subprocess.run(
                ["where.exe", lookup_term],
                capture_output=True,
                text=True,
                env=get_env(),
            )

            logger.debug(f"where.exe returncode: {where_result.returncode}")
            logger.debug(f"where.exe stdout: {where_result.stdout}")
            logger.debug(f"where.exe stderr: {where_result.stderr}")

            if where_result.returncode == 0:
                for path in where_result.stdout.splitlines():
                    path = path.strip()
                    if not path:
                        continue

                    logger.debug(f"Checking path: {path}")
                    path_lower = path.lower()

                    if not path_lower.endswith((".exe", ".bat", ".cmd")):
                        logger.debug(f"Skipping non-executable path: {path}")
                        continue

                    if "\\safety\\" in path_lower and path_lower.endswith(
                        f"{name}.bat"
                    ):
                        logger.debug(f"Skipping Safety wrapper: {path}")
                        continue

                    return path

        logger.debug(
            f"No unwrapped command found on Windows, returning bare name: {name}"
        )
        return name

    fallback = shutil.which(name, path=get_path()) or name
    logger.debug(f"Using fallback (shutil.which or name): {fallback}")
    return fallback



"""
Test suite for the base ToolCommandLineParser functionality.
"""

import pytest

from safety.tool.base import ToolCommandLineParser
from safety.tool.intents import ToolIntentionType


class MockParser(ToolCommandLineParser):
    """Mock parser for testing base functionality"""

    def get_tool_name(self) -> str:
        return "test-tool"

    def get_command_hierarchy(self):
        return {
            "install": ToolIntentionType.ADD_PACKAGE,
            "remove": ToolIntentionType.REMOVE_PACKAGE,
            "pip": {
                "install": ToolIntentionType.ADD_PACKAGE,
                "uninstall": ToolIntentionType.REMOVE_PACKAGE,
            },
        }

    def get_known_flags(self):
        return {
            "global": {"verbose", "v", "quiet", "q", "help", "h"},
            "install": {"upgrade", "no-deps", "user"},
            "pip.install": {"upgrade", "force-reinstall", "user"},
        }


@pytest.mark.unit
class TestToolCommandLineParser:
    """Test base parser functionality"""

    def setup_method(self):
        """Set up test fixtures"""
        self.parser = MockParser()

    def test_parser_known_flag_without_value_does_not_consume_next_arg(self):
        """Ensure flags like --upgrade don't consume package names"""
        intention = self.parser.parse(["install", "--upgrade", "requests"])

        assert intention is not None
        assert intention.intention_type == ToolIntentionType.ADD_PACKAGE
        assert len(intention.packages) == 1
        assert intention.packages[0].name == "requests"
        assert "upgrade" in intention.options
        assert intention.options["upgrade"]["value"] is True

    def test_parser_option_equals_value_form_is_parsed(self):
        """Test --index-url=https://... format"""
        intention = self.parser.parse(
            ["install", "--index-url=https://pypi.org/simple", "requests"]
        )

        assert intention is not None
        assert "index-url" in intention.options
        assert intention.options["index-url"]["value"] == "https://pypi.org/simple"
        assert len(intention.packages) == 1
        assert intention.packages[0].name == "requests"

    def test_parser_option_space_value_form_is_parsed(self):
        """Test --index-url https://... format"""
        intention = self.parser.parse(
            ["install", "--index-url", "https://pypi.org/simple", "requests"]
        )

        assert intention is not None
        assert "index-url" in intention.options
        assert intention.options["index-url"]["value"] == "https://pypi.org/simple"
        assert len(intention.packages) == 1
        assert intention.packages[0].name == "requests"

    def test_parser_unknown_dash_arg_stored_as_unknown(self):
        """Unknown dash args recorded with arg_index"""
        intention = self.parser.parse(["install", "--unknown-option", "requests"])

        assert intention is not None
        assert "unknown-option" in intention.options
        # Unknown option followed by non-dash arg will consume the next arg as value
        assert intention.options["unknown-option"]["value"] == "requests"
        assert "arg_index" in intention.options["unknown-option"]
        assert len(intention.packages) == 0  # requests was consumed as option value

    def test_parser_packages_extracted_with_arg_index_and_spec(self):
        """Package specs parsed correctly when valid"""
        intention = self.parser.parse(["install", "requests>=2.25.1", "flask==2.0.1"])

        assert intention is not None
        assert len(intention.packages) == 2

        # First package
        pkg1 = intention.packages[0]
        assert pkg1.name == "requests"
        assert pkg1.version_constraint == ">=2.25.1"
        assert pkg1.original_text == "requests>=2.25.1"
        assert pkg1.arg_index == 1

        # Second package
        pkg2 = intention.packages[1]
        assert pkg2.name == "flask"
        assert pkg2.version_constraint == "==2.0.1"
        assert pkg2.original_text == "flask==2.0.1"
        assert pkg2.arg_index == 2

    def test_parser_multilevel_command_hierarchy_parsed(self):
        """Test pip install command recognition"""
        intention = self.parser.parse(["pip", "install", "requests"])

        assert intention is not None
        assert intention.command_chain == ["pip", "install"]
        assert intention.command == "pip install"
        assert intention.intention_type == ToolIntentionType.ADD_PACKAGE
        assert len(intention.packages) == 1
        assert intention.packages[0].name == "requests"

    def test_parser_command_specific_flags_override_global(self):
        """Command-specific flags take precedence over global flags"""
        intention = self.parser.parse(["pip", "install", "--upgrade", "requests"])

        assert intention is not None
        assert "upgrade" in intention.options
        assert intention.options["upgrade"]["value"] is True
        assert len(intention.packages) == 1
        assert intention.packages[0].name == "requests"

    def test_parser_invalid_command_returns_none(self):
        """Invalid commands return None"""
        intention = self.parser.parse(["invalid-command", "requests"])

        assert intention is None

    def test_parser_empty_args_returns_none(self):
        """Empty arguments return None"""
        intention = self.parser.parse([])

        assert intention is None

    def test_parser_non_package_intentions_dont_parse_packages(self):
        """Non-package intentions store args as unknown"""
        # Add a non-package intention to our mock
        self.parser.get_command_hierarchy = lambda: {
            "sync": ToolIntentionType.SYNC_PACKAGES,
            "install": ToolIntentionType.ADD_PACKAGE,
        }

        intention = self.parser.parse(["sync", "some-arg"])

        assert intention is not None
        assert intention.intention_type == ToolIntentionType.SYNC_PACKAGES
        assert len(intention.packages) == 0
        assert "unknown_0" in intention.options
        assert intention.options["unknown_0"]["value"] == "some-arg"

    def test_parser_raw_args_preserved(self):
        """Original raw arguments are preserved"""
        args = ["install", "--upgrade", "requests>=2.0"]
        intention = self.parser.parse(args)

        assert intention is not None
        assert intention.raw_args == args
        # Ensure original args are not modified
        assert args == ["install", "--upgrade", "requests>=2.0"]

    def test_parser_arg_indices_correct(self):
        """Argument indices are correctly tracked"""
        intention = self.parser.parse(
            ["install", "--verbose", "--index-url", "https://pypi.org", "requests"]
        )

        assert intention is not None

        # Option indices
        assert intention.options["verbose"]["arg_index"] == 1
        assert intention.options["index-url"]["arg_index"] == 2
        assert intention.options["index-url"]["value_index"] == 3

        # Package index
        assert intention.packages[0].arg_index == 4

    def test_parser_mixed_flags_and_options(self):
        """Mix of boolean flags and value options parsed correctly"""
        intention = self.parser.parse(
            [
                "install",
                "--upgrade",
                "--index-url",
                "https://custom.pypi.org",
                "--user",
                "requests",
                "flask",
            ]
        )

        assert intention is not None

        # Boolean flags
        assert intention.options["upgrade"]["value"] is True
        assert intention.options["user"]["value"] is True

        # Value option
        assert intention.options["index-url"]["value"] == "https://custom.pypi.org"

        # Packages
        assert len(intention.packages) == 2
        assert intention.packages[0].name == "requests"
        assert intention.packages[1].name == "flask"

    def test_parser_tool_name_set(self):
        """Parser correctly sets tool name in intention"""
        intention = self.parser.parse(["install", "requests"])

        assert intention is not None
        assert intention.tool == "test-tool"



"""
Tests for event emission utilities.
"""

import pytest
import uuid
from unittest.mock import MagicMock, patch

from safety_schemas.models.events import EventType
from safety.events.utils.emission import emit_diff_operations
from safety.tool.environment_diff import PackageLocation
from safety_schemas.models.events.types import ToolType


@pytest.mark.unit
class TestEmitDiffOperations:
    """
    Test suite for emit_diff_operations function.
    """

    def setup_method(self):
        """
        Set up test fixtures.
        """
        self.event_bus = MagicMock()
        self.ctx = MagicMock()
        self.ctx.obj = MagicMock()
        self.ctx.obj.correlation_id = str(uuid.uuid4())

    @pytest.mark.parametrize(
        "added, removed, updated, tool_path, expected_calls",
        [
            (
                {
                    PackageLocation(
                        name="requests",
                        location="/usr/local/lib/python3.9/site-packages",
                    ): "2.28.1",
                    PackageLocation(
                        name="urllib3",
                        location="/home/user/.local/lib/python3.9/site-packages",
                    ): "1.26.12",
                },
                {
                    PackageLocation(
                        name="old-package", location="/usr/lib/python3.9/site-packages"
                    ): "1.0.0"
                },
                {
                    PackageLocation(
                        name="numpy", location="/usr/local/lib/python3.9/site-packages"
                    ): ("1.21.0", "1.23.5")
                },
                "/usr/bin/pip",
                4,
            ),
            (
                {
                    PackageLocation(
                        name="requests", location="C:\\Python39\\Lib\\site-packages"
                    ): "2.28.1"
                },
                {
                    PackageLocation(
                        name="old-pkg",
                        location="C:\\Users\\User\\AppData\\Local\\Programs\\Python\\Python39\\Lib\\site-packages",
                    ): "1.0.0"
                },
                {
                    PackageLocation(
                        name="numpy", location="C:\\Python39\\Lib\\site-packages"
                    ): ("1.21.0", "1.23.5")
                },
                "C:\\Python39\\Scripts\\pip.exe",
                3,
            ),
        ],
    )
    def test_emit_diff_operations_with_multiple_operations(
        self, added, removed, updated, tool_path, expected_calls
    ):
        """
        The emit_diff_operations function should emit the correct number of events
        based on the number of added, removed, and updated packages.
        """
        by_tool = ToolType.PIP

        emit_diff_operations(
            self.event_bus,
            self.ctx,
            added=added,
            removed=removed,
            updated=updated,
            tool_path=tool_path,
            by_tool=by_tool,
        )

        # Basic count and common payload assertions
        assert self.event_bus.emit.call_count == expected_calls

        # Group events by type (order is not guaranteed)
        installed_events = []
        uninstalled_events = []
        updated_events = []

        for call in self.event_bus.emit.call_args_list:
            event = call[0][0]
            assert event.payload.tool == by_tool
            assert event.payload.tool_path == tool_path

            if event.type == EventType.PACKAGE_INSTALLED:
                installed_events.append(event)
            elif event.type == EventType.PACKAGE_UNINSTALLED:
                uninstalled_events.append(event)
            elif event.type == EventType.PACKAGE_UPDATED:
                updated_events.append(event)

        # Assert counts by type
        assert len(installed_events) == len(added)
        assert len(uninstalled_events) == len(removed)
        assert len(updated_events) == len(updated)

        # Validate payload fields for installed events
        expected_installed = {(pl.name, v, pl.location) for pl, v in added.items()}
        observed_installed = {
            (e.payload.package_name, e.payload.version, e.payload.location)
            for e in installed_events
        }
        assert observed_installed == expected_installed

        # Validate payload fields for uninstalled events
        expected_uninstalled = {(pl.name, v, pl.location) for pl, v in removed.items()}
        observed_uninstalled = {
            (e.payload.package_name, e.payload.version, e.payload.location)
            for e in uninstalled_events
        }
        assert observed_uninstalled == expected_uninstalled

        # Validate payload fields for updated events (including tuple order)
        expected_updated = {
            (pl.name, pl.location, pv, cv) for pl, (pv, cv) in updated.items()
        }
        observed_updated = {
            (
                e.payload.package_name,
                e.payload.location,
                e.payload.previous_version,
                e.payload.current_version,
            )
            for e in updated_events
        }
        assert observed_updated == expected_updated

    def test_emit_diff_operations_with_empty_locations(self):
        """
        Test emit_diff_operations with empty or None locations.
        """
        # Arrange
        added = {
            PackageLocation(name="pkg-no-location", location=""): "1.0.0",
            PackageLocation(name="pkg-none-location", location=None or ""): "2.0.0",
        }
        removed = {}
        updated = {}
        tool_path = "/usr/bin/pip"
        by_tool = ToolType.PIP

        # Act
        emit_diff_operations(
            self.event_bus,
            self.ctx,
            added=added,
            removed=removed,
            updated=updated,
            tool_path=tool_path,
            by_tool=by_tool,
        )

        # Assert - Should still emit events with empty locations
        assert self.event_bus.emit.call_count == 2

        # Verify events are emitted for packages with empty locations
        for call in self.event_bus.emit.call_args_list:
            event = call[0][0]
            assert event.payload.location == ""
            assert event.payload.tool == by_tool
            assert event.type == EventType.PACKAGE_INSTALLED

    def test_emit_diff_operations_without_correlation_id(self):
        """
        Test emit_diff_operations generates correlation_id when not present.
        """
        # Arrange
        self.ctx.obj.correlation_id = None
        added = {
            PackageLocation(
                name="test-pkg", location="/usr/local/lib/python3.9/site-packages"
            ): "1.0.0"
        }
        removed = {}
        updated = {}
        tool_path = "/usr/bin/pip"
        by_tool = ToolType.PIP

        # Act
        with patch(
            "uuid.uuid4", return_value=uuid.UUID("12345678-1234-5678-1234-567812345678")
        ):
            emit_diff_operations(
                self.event_bus,
                self.ctx,
                added=added,
                removed=removed,
                updated=updated,
                tool_path=tool_path,
                by_tool=by_tool,
            )

        # Assert - Should generate and assign correlation ID
        assert self.ctx.obj.correlation_id == "12345678-1234-5678-1234-567812345678"

    @pytest.mark.parametrize(
        "preset_id, expected_id",
        [
            ("preset-id", "preset-id"),
            (None, "12345678-1234-5678-1234-567812345678"),
        ],
    )
    def test_emit_diff_operations_correlation_id_propagation(
        self, preset_id, expected_id
    ):
        """
        Events should carry the existing correlation_id if preset; otherwise a new one is generated
        and attached to the context and emitted events.
        """
        # Arrange
        self.ctx.obj.correlation_id = preset_id
        added = {PackageLocation(name="pkg", location="/site-packages"): "1.0.0"}

        if preset_id is None:
            uuid_patch = patch(
                "uuid.uuid4",
                return_value=uuid.UUID("12345678-1234-5678-1234-567812345678"),
            )
        else:
            uuid_patch = patch("uuid.uuid4")  # no-op

        with uuid_patch:
            emit_diff_operations(
                self.event_bus,
                self.ctx,
                added=added,
                removed={},
                updated={},
                tool_path="/usr/bin/pip",
                by_tool=ToolType.PIP,
            )

        # Assert every emitted event has the expected correlation id
        assert self.ctx.obj.correlation_id == expected_id
        for call in self.event_bus.emit.call_args_list:
            event = call[0][0]
            assert event.correlation_id == expected_id

    def test_emit_diff_operations_with_no_changes(self):
        """
        Test emit_diff_operations with no package changes.
        """
        # Arrange
        added = {}
        removed = {}
        updated = {}
        tool_path = "/usr/bin/pip"
        by_tool = ToolType.PIP

        # Act
        emit_diff_operations(
            self.event_bus,
            self.ctx,
            added=added,
            removed=removed,
            updated=updated,
            tool_path=tool_path,
            by_tool=by_tool,
        )

        # Assert - Should not emit any events for empty changes
        self.event_bus.emit.assert_not_called()

    @pytest.mark.parametrize(
        "tool_type, tool_path",
        [
            (ToolType.PIP, "/usr/bin/pip"),
            (ToolType.POETRY, "/usr/local/bin/poetry"),
            (ToolType.UV, "/home/user/.cargo/bin/uv"),
        ],
    )
    def test_emit_diff_operations_with_different_tools(self, tool_type, tool_path):
        """
        Test emit_diff_operations with different tool types.
        """
        # Arrange
        added = {
            PackageLocation(
                name="test-pkg", location="/usr/local/lib/python3.9/site-packages"
            ): "1.0.0"
        }

        # Act
        emit_diff_operations(
            self.event_bus,
            self.ctx,
            added=added,
            removed={},
            updated={},
            tool_path=tool_path,
            by_tool=tool_type,
        )

        # Assert - Tool type should be correctly set
        assert self.event_bus.emit.call_count == 1
        package_event = self.event_bus.emit.call_args_list[0][0][0]
        assert package_event.payload.tool_path == tool_path
        assert package_event.payload.tool == tool_type

    def test_emit_diff_operations_creates_correct_payload_structure(self):
        """
        Test that emit_diff_operations creates events with correct payload structure.
        """
        # Arrange
        package_loc = PackageLocation(
            name="test-pkg", location="/usr/local/lib/python3.9/site-packages"
        )
        added = {package_loc: "1.0.0"}
        tool_path = "/usr/bin/pip"
        by_tool = ToolType.PIP

        # Act
        emit_diff_operations(
            self.event_bus,
            self.ctx,
            added=added,
            removed={},
            updated={},
            tool_path=tool_path,
            by_tool=by_tool,
        )

        # Assert - Check the event was emitted with correct structure
        assert self.event_bus.emit.call_count == 1
        event_call = self.event_bus.emit.call_args_list[0]
        event = event_call[0][0]

        # Check event type and payload structure
        assert event.type == EventType.PACKAGE_INSTALLED
        assert event.payload.package_name == "test-pkg"
        assert event.payload.version == "1.0.0"
        assert event.payload.location == "/usr/local/lib/python3.9/site-packages"
        assert event.payload.tool_path == tool_path
        assert event.payload.tool == by_tool
